{-# OPTIONS_GHC -fglasgow-exts #-}

module ProofChecker where {

import qualified Show;
import qualified Term;
import qualified Loop;
import qualified DpTransImpl;
import qualified List;
import qualified ErrorMonad;
import qualified RedPairImpl;
import qualified DepGraph;
import qualified Option;
import qualified SemanticLabeling;
import qualified RootLabeling;
import qualified SubtermCriterionImpl;
import qualified SCT;
import qualified Trs;
import qualified DpFramework;
import qualified Nat;
import qualified Eithera;

debug :: forall a. [Char] -> [Char] -> a -> a;
debug i t x = x;

data DPProof a b c = PisEmpty
  | RedPairProc (RedPairImpl.Redpair (SemanticLabeling.Lab a b))
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      (ProofChecker.DPProof a b c)
  | RedPairUrProc (RedPairImpl.Redpair (SemanticLabeling.Lab a b))
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      (ProofChecker.DPProof a b c)
  | DepGraphProc
      [(Maybe (ProofChecker.DPProof a b c),
         [(Term.Term (SemanticLabeling.Lab a b) c,
            Term.Term (SemanticLabeling.Lab a b) c)])]
  | MonoRedPairProc (RedPairImpl.Redpair (SemanticLabeling.Lab a b))
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      (ProofChecker.DPProof a b c)
  | MonoRedPairUrProc (RedPairImpl.Redpair (SemanticLabeling.Lab a b))
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      (ProofChecker.DPProof a b c)
  | UnlabProc
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      (ProofChecker.DPProof a b c)
  | SemlabProc (SemanticLabeling.Sl_inter a b)
      (SemanticLabeling.Lab a b ->
        [Nat.Nat] -> Eithera.Eithera b [(SemanticLabeling.Lab a b)])
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      (ProofChecker.DPProof a b c)
  | RootlabProc
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      (ProofChecker.DPProof a b c)
  | SpscProc (SubtermCriterionImpl.SpL (SemanticLabeling.Lab a b))
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      (ProofChecker.DPProof a b c)
  | SizeChangeSubTerm
      [((Term.Term (SemanticLabeling.Lab a b) c,
          Term.Term (SemanticLabeling.Lab a b) c),
         ([(Nat.Nat, Nat.Nat)], [(Nat.Nat, Nat.Nat)]))]
  | SizeChangeRedPair (RedPairImpl.Redpair (SemanticLabeling.Lab a b))
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      [((Term.Term (SemanticLabeling.Lab a b) c,
          Term.Term (SemanticLabeling.Lab a b) c),
         ([(Nat.Nat, Nat.Nat)], [(Nat.Nat, Nat.Nat)]))];

data TRSProof a b c =
  DPTrans
    [(Term.Term (Trs.Shp (SemanticLabeling.Lab a b)) c,
       Term.Term (Trs.Shp (SemanticLabeling.Lab a b)) c)]
    (ProofChecker.DPProof (Trs.Shp (SemanticLabeling.Lab a b)) b c)
  | RuleRemoval (RedPairImpl.Redpair (SemanticLabeling.Lab a b))
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      (ProofChecker.TRSProof a b c)
  | Unlab [(Term.Term (SemanticLabeling.Lab a b) c,
             Term.Term (SemanticLabeling.Lab a b) c)]
      (ProofChecker.TRSProof a b c)
  | Semlab (SemanticLabeling.Sl_inter a b)
      (SemanticLabeling.Lab a b ->
        [Nat.Nat] -> Eithera.Eithera b [(SemanticLabeling.Lab a b)])
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      (ProofChecker.TRSProof a b c)
  | Rootlab
      [(Term.Term (SemanticLabeling.Lab a b) c,
         Term.Term (SemanticLabeling.Lab a b) c)]
      (ProofChecker.TRSProof a b c)
  | RisEmpty;

data TRSDisproof a b =
  Loop ([(Term.Term a b)], (Term.Ctxt a b, [(Term.Term a b, Term.Term a b)]))
  | NotWellFormed;

data CertificationResult = Certified Bool | Error [Char];

indent :: [[Char]] -> [[Char]];
indent xs = Show.newline : List.mapa (List.append [' ', ' ']) xs;

data CertificationProblem a b c =
  CertificationProblem Bool [(Term.Term a c, Term.Term a c)]
    (Eithera.Eithera (ProofChecker.TRSProof a b c)
      (ProofChecker.TRSDisproof a c));

checkTRSDisproof ::
  forall a b.
    (Eq a, Show.Sshow a, Eq b,
      Show.Sshow b) => [Char] ->
                         [(Term.Term a b, Term.Term a b)] ->
                           ProofChecker.TRSDisproof a b ->
                             Eithera.Eithera [[Char]] ();
checkTRSDisproof i r (ProofChecker.Loop l) =
  ProofChecker.debug i ['L', 'o', 'o', 'p']
    (ErrorMonad.catcha (Loop.checkLoop r l)
      (\ x ->
        ErrorMonad.errora
          (List.append i
             [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ', 'l', 'o',
               'o', 'p', ' ', 'p', 'r', 'o', 'o', 'f']
            : x)));
checkTRSDisproof i r ProofChecker.NotWellFormed =
  ProofChecker.debug i
    ['n', 'o', 't', ' ', 'w', 'e', 'l', 'l', '-', 'f', 'o', 'r', 'm', 'e', 'd']
    (ErrorMonad.catcha (DpTransImpl.checkNotWellFormed r)
      (\ x ->
        ErrorMonad.errora
          (List.append i
             [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ', 'n', 'o',
               't', '-', 'w', 'e', 'l', 'l', '-', 'f', 'o', 'r', 'm', 'e', 'd',
               ' ', 'p', 'r', 'o', 'o', 'f']
            : x)));

checkDPProof ::
  forall a b.
    (Eq a, Show.Sshow a, Eq b,
      Show.Sshow b) => [Char] ->
                         [(Term.Term (SemanticLabeling.Lab a b) [Char],
                            Term.Term (SemanticLabeling.Lab a b) [Char])]
                           -> [(Term.Term (SemanticLabeling.Lab a b) [Char],
                                 Term.Term (SemanticLabeling.Lab a b) [Char])]
                                -> Nat.Nat ->
                                     Maybe (SemanticLabeling.Lab a b ->
     [(Term.Term (SemanticLabeling.Lab a b) [Char],
        Term.Term (SemanticLabeling.Lab a b) [Char])])
                                       -> Maybe (SemanticLabeling.Lab a b ->
          [(Term.Term (SemanticLabeling.Lab a b) [Char],
             Term.Term (SemanticLabeling.Lab a b) [Char])])
    -> ProofChecker.DPProof a b [Char] -> Eithera.Eithera [[Char]] ();
checkDPProof i p r m rm rrm ProofChecker.PisEmpty =
  ProofChecker.debug i ['P', ' ', 'i', 's', ' ', 'e', 'm', 'p', 't', 'y']
    (case p of {
      [] -> ErrorMonad.returna ();
      x : xa ->
        ErrorMonad.errora
          (List.append i
             [':', ' ', 'P', ' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'e', 'm',
               'p', 't', 'y', ' ', 'i', 'n', ' ', 't', 'h', 'e', ' ', 'f', 'o',
               'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'D', 'P', '-', 'p', 'r',
               'o', 'b', 'l', 'e', 'm', ' ']
            : Trs.showDPP p r);
    });
checkDPProof i p r m rm rrm (ProofChecker.RedPairProc redp rP prf) =
  ProofChecker.debug i ['R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c']
    (ErrorMonad.bind
      (ErrorMonad.catcha (RedPairImpl.create_redpair_proc redp p rP r)
        (\ x ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ',
                 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h', 'e',
                 ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n', ' ', 'p',
                 'a', 'i', 'r', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o',
                 'r', ' ', 't', 'o', ' ', 's', 'w', 'i', 't', 'c', 'h', ' ',
                 'f', 'r', 'o', 'm', ' ']
              : List.append (Trs.showDPP p r)
                  (['t', 'o', ' ', 't', 'h', 'e', ' ', 'D', 'P', ' ', 'p', 'r',
                     'o', 'b', 'l', 'e', 'm', ' ']
                    : List.append (Trs.showDPP rP r) x))))
      (\ x ->
        ErrorMonad.catcha
          (ProofChecker.checkDPProof (List.append i ['.', '1']) rP r m rm rrm
            prf)
          (\ xa ->
            ErrorMonad.errora
              (List.append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 'r', 'e', 'd', 'u', 'c', 't',
                   'i', 'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p', 'r', 'o',
                   'c', 'e', 's', 's', 'o', 'r']
                : ProofChecker.indent xa))));
checkDPProof i p r m rm rrm (ProofChecker.RedPairUrProc redp rP ur prf) =
  ProofChecker.debug i
    ['R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c', ' ', '(', 'u', 's',
      'a', 'b', 'l', 'e', ' ', 'r', 'u', 'l', 'e', 's', ' ', '+', ' ', 'a', 'f',
      ')']
    (ErrorMonad.bind
      (ErrorMonad.catcha
        (RedPairImpl.create_redpair_ur_af_proc redp p rP r rm ur)
        (\ x ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ',
                 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h', 'e',
                 ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n', ' ', 'p',
                 'a', 'i', 'r', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o',
                 'r', ' ', '(', 'w', 'i', 't', 'h', ' ', 'u', 's', 'a', 'b',
                 'l', 'e', ' ', 'r', 'u', 'l', 'e', 's', ' ', 'a', 'n', 'd',
                 ' ', 'a', 'f', ')', ' ', 't', 'o', ' ', 's', 'w', 'i', 't',
                 'c', 'h', ' ', 'f', 'r', 'o', 'm', ' ']
              : List.append (Trs.showDPP p r)
                  (['t', 'o', ' ', 't', 'h', 'e', ' ', 'D', 'P', ' ', 'p', 'r',
                     'o', 'b', 'l', 'e', 'm', ' ']
                    : List.append (Trs.showDPP rP r) x))))
      (\ x ->
        ErrorMonad.catcha
          (ProofChecker.checkDPProof (List.append i ['.', '1']) rP r m rm rrm
            prf)
          (\ xa ->
            ErrorMonad.errora
              (List.append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 'r', 'e', 'd', 'u', 'c', 't',
                   'i', 'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p', 'r', 'o',
                   'c', 'e', 's', 's', 'o', 'r']
                : ProofChecker.indent xa))));
checkDPProof i p r m rm rrm (ProofChecker.DepGraphProc edpts) =
  ProofChecker.debug i
    ['D', 'e', 'p', 'G', 'r', 'a', 'p', 'h', 'P', 'r', 'o', 'c']
    (ErrorMonad.bind
      (ErrorMonad.catcha
        (DepGraph.checkDepGraphProc p
          (List.mapa
            (\ x ->
              ((case fst x of {
                 Nothing -> False;
                 Just xa -> True;
               }),
                snd x))
            edpts)
          rm rrm)
        (\ x ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'i', 'l', 'e',
                 ' ', 't', 'r', 'y', 'i', 'n', 'g', ' ', 't', 'o', ' ', 'p',
                 'e', 'r', 'f', 'o', 'r', 'm', ' ', 'S', 'C', 'C', '-', 'd',
                 'e', 'c', 'o', 'm', 'p', 'o', 's', 'i', 't', 'i', 'o', 'n',
                 ' ', ' ', 'o', 'n', ' ']
              : List.append (Trs.showDPP p r) x)))
      (\ x ->
        ErrorMonad.catcha
          (ErrorMonad.checkAllMIndex
            (\ j rp ->
              (case fst rp of {
                Nothing -> j;
                Just prf -> Nat.Suc j;
              }))
            (\ j rp ->
              (case fst rp of {
                Nothing -> ErrorMonad.returna ();
                Just a ->
                  ProofChecker.checkDPProof
                    (List.append (List.append i ['.']) (Show.sshow_nat j))
                    (snd rp) r m rm rrm a;
              }))
            edpts)
          (\ xa ->
            ErrorMonad.errora
              (List.append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 'd', 'e', 'p', 'e', 'n', 'd',
                   'e', 'n', 'c', 'y', ' ', 'g', 'r', 'a', 'p', 'h', ' ', 'p',
                   'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
                : ProofChecker.indent xa))));
checkDPProof i p r m rm rrm (ProofChecker.MonoRedPairProc redp rP rR prf) =
  ProofChecker.debug i
    ['M', 'o', 'n', 'o', 'R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c']
    (ErrorMonad.bind
      (ErrorMonad.catcha (RedPairImpl.create_mono_redpair_proc redp p rP r rR)
        (\ x ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ',
                 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h', 'e',
                 ' ', 'm', 'o', 'n', 'o', 't', 'o', 'n', 'i', 'c', ' ', 'r',
                 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n', ' ', 'p', 'a', 'i',
                 'r', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r', ' ',
                 't', 'o', ' ', 's', 'w', 'i', 't', 'c', 'h', ' ', 'f', 'r',
                 'o', 'm', ' ']
              : List.append (Trs.showDPP p r)
                  (['t', 'o', ' ', 't', 'h', 'e', ' ', 'D', 'P', ' ', 'p', 'r',
                     'o', 'b', 'l', 'e', 'm', ' ']
                    : List.append (Trs.showDPP rP rR) x))))
      (\ x ->
        ErrorMonad.catcha
          (ProofChecker.checkDPProof (List.append i ['.', '1']) rP rR m
            (DpFramework.computeRuleMap rR)
            (DpFramework.computeRuleMap (DpFramework.reverse_trs rR)) prf)
          (\ xa ->
            ErrorMonad.errora
              (List.append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 'm', 'o', 'n', 'o', 't', 'o',
                   'n', 'i', 'c', ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i', 'o',
                   'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p', 'r', 'o', 'c', 'e',
                   's', 's', 'o', 'r']
                : ProofChecker.indent xa))));
checkDPProof i p r m rm rrm (ProofChecker.MonoRedPairUrProc redp rP rR ur prf) =
  ProofChecker.debug i
    ['M', 'o', 'n', 'o', 'R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c',
      ' ', '(', 'u', 's', 'a', 'b', 'l', 'e', ' ', 'r', 'u', 'l', 'e', 's', ')']
    (ErrorMonad.bind
      (ErrorMonad.catcha
        (RedPairImpl.create_mono_redpair_ur_proc redp p rP r rm rR ur)
        (\ x ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ',
                 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h', 'e',
                 ' ', 'm', 'o', 'n', 'o', 't', 'o', 'n', 'i', 'c', ' ', 'r',
                 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n', ' ', 'p', 'a', 'i',
                 'r', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r', ' ',
                 '(', 'w', 'i', 't', 'h', ' ', 'u', 's', 'a', 'b', 'l', 'e',
                 ' ', 'r', 'u', 'l', 'e', 's', ')', ' ', 't', 'o', ' ', 's',
                 'w', 'i', 't', 'c', 'h', ' ', 'f', 'r', 'o', 'm', ' ']
              : List.append (Trs.showDPP p r)
                  (['t', 'o', ' ', 't', 'h', 'e', ' ', 'D', 'P', ' ', 'p', 'r',
                     'o', 'b', 'l', 'e', 'm', ' ']
                    : List.append (Trs.showDPP rP rR) x))))
      (\ x ->
        ErrorMonad.catcha
          (ProofChecker.checkDPProof (List.append i ['.', '1']) rP rR m
            (DpFramework.computeRuleMap rR)
            (DpFramework.computeRuleMap (DpFramework.reverse_trs rR)) prf)
          (\ xa ->
            ErrorMonad.errora
              (List.append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 'm', 'o', 'n', 'o', 't', 'o',
                   'n', 'i', 'c', ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i', 'o',
                   'n', ' ', 'p', 'a', 'i', 'r', ' ', '(', 'w', 'i', 't', 'h',
                   ' ', 'u', 's', 'a', 'b', 'l', 'e', ' ', 'r', 'u', 'l', 'e',
                   's', ')', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
                : ProofChecker.indent xa))));
checkDPProof i p r (Nat.Suc m) rm rrm (ProofChecker.UnlabProc ulP ulR prf) =
  ProofChecker.debug i ['U', 'n', 'l', 'a', 'b', 'P', 'r', 'o', 'c']
    (ErrorMonad.bind (SemanticLabeling.check_unlab_proc p r ulP ulR)
      (\ x ->
        ErrorMonad.catcha
          (ProofChecker.checkDPProof (List.append i ['.', '1']) ulP ulR m
            (DpFramework.computeRuleMap ulR)
            (DpFramework.computeRuleMap (DpFramework.reverse_trs ulR)) prf)
          (\ xa ->
            ErrorMonad.errora
              (List.append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 'u', 'n', 'l', 'a', 'b', 'e',
                   'l', 'i', 'n', 'g', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's',
                   'o', 'r']
                : ProofChecker.indent xa))));
checkDPProof i p r Nat.Zero_nat rm rrm (ProofChecker.UnlabProc ulP ulR prf) =
  ErrorMonad.errora
    [['u', 'n', 'l', 'a', 'b', 'e', 'l', 'i', 'n', 'g', ' ', 'n', 'o', 't', ' ',
       'a', 'l', 'l', 'o', 'w', 'e', 'd', ' ', 'd', 'u', 'e', ' ', 't', 'o',
       ' ', 'l', 'a', 'b', 'e', 'l', '-', 'd', 'e', 'p', 't', 'h', ' ', 'o',
       'f', ' ', '0']];
checkDPProof i p r m rm rrm (ProofChecker.SemlabProc sli l lP lR prf) =
  ProofChecker.debug i ['S', 'e', 'm', 'l', 'a', 'b', 'P', 'r', 'o', 'c']
    (ErrorMonad.bind
      (ErrorMonad.catcha (SemanticLabeling.create_sem_lab_proc sli l p r lP lR)
        (\ x ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ',
                 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h', 'e',
                 ' ', 's', 'e', 'm', 'a', 'n', 't', 'i', 'c', ' ', 'l', 'a',
                 'b', 'e', 'l', 'i', 'n', 'g', ' ', 'p', 'r', 'o', 'c', 'e',
                 's', 's', 'o', 'r']
              : x)))
      (\ x ->
        ErrorMonad.catcha
          (ProofChecker.checkDPProof (List.append i ['.', '1']) lP lR
            (Nat.Suc m) (DpFramework.computeRuleMap lR)
            (DpFramework.computeRuleMap (DpFramework.reverse_trs lR)) prf)
          (\ xa ->
            ErrorMonad.errora
              (List.append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 's', 'e', 'm', 'a', 'n', 't',
                   'i', 'c', ' ', 'l', 'a', 'b', 'e', 'l', 'i', 'n', 'g', ' ',
                   'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
                : ProofChecker.indent xa))));
checkDPProof i p r m rm rrm (ProofChecker.RootlabProc lP lR prf) =
  ProofChecker.debug i ['R', 'o', 'o', 't', 'l', 'a', 'b', 'P', 'r', 'o', 'c']
    (ErrorMonad.bind
      (ErrorMonad.catcha (RootLabeling.rl_proc p r lP lR)
        (\ x ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ',
                 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h', 'e',
                 ' ', 'r', 'o', 'o', 't', ' ', 'l', 'a', 'b', 'e', 'l', 'i',
                 'n', 'g', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
              : x)))
      (\ x ->
        ErrorMonad.catcha
          (ProofChecker.checkDPProof (List.append i ['.', '1']) lP lR
            (Nat.Suc m) (DpFramework.computeRuleMap lR)
            (DpFramework.computeRuleMap (DpFramework.reverse_trs lR)) prf)
          (\ xa ->
            ErrorMonad.errora
              (List.append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 'r', 'o', 'o', 't', '-', 'l',
                   'a', 'b', 'e', 'l', 'i', 'n', 'g', ' ', 'p', 'r', 'o', 'c',
                   'e', 's', 's', 'o', 'r']
                : ProofChecker.indent xa))));
checkDPProof i p r m rm rrm (ProofChecker.SpscProc sp rP prf) =
  ProofChecker.debug i ['S', 'u', 'b', 't', 'e', 'r', 'm', 'P', 'r', 'o', 'c']
    (ErrorMonad.bind
      (ErrorMonad.catcha (SubtermCriterionImpl.create_spsc_proc sp p rP r)
        (\ x ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ',
                 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h', 'e',
                 ' ', 's', 'u', 'b', 't', 'e', 'r', 'm', ' ', 'c', 'r', 'i',
                 't', 'e', 'r', 'i', 'o', 'n', ' ', 'p', 'r', 'o', 'c', 'e',
                 's', 's', 'o', 'r']
              : x)))
      (\ x ->
        ErrorMonad.catcha
          (ProofChecker.checkDPProof (List.append i ['.', '1']) rP r m rm rrm
            prf)
          (\ xa ->
            ErrorMonad.errora
              (List.append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 's', 'u', 'b', 't', 'e', 'r',
                   'm', ' ', 'c', 'r', 'i', 't', 'e', 'r', 'i', 'o', 'n', ' ',
                   'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
                : ProofChecker.indent xa))));
checkDPProof i p r m rm rrm (ProofChecker.SizeChangeSubTerm graphs) =
  ProofChecker.debug i
    ['S', 'i', 'z', 'e', 'C', 'h', 'a', 'n', 'g', 'e', 'S', 'u', 'b', 'T', 'e',
      'r', 'm']
    (SCT.check_sct_subterm_proc p graphs r rm rrm);
checkDPProof i p r m rm rrm (ProofChecker.SizeChangeRedPair redp ur graphs) =
  ProofChecker.debug i
    ['S', 'i', 'z', 'e', 'C', 'h', 'a', 'n', 'g', 'e', 'R', 'e', 'd', 'P', 'a',
      'i', 'r']
    (SCT.create_sct_ur_af_proc redp p r rm rrm ur graphs);

checkTRSProof ::
  forall a b.
    (Eq a, Show.Sshow a, Eq b,
      Show.Sshow b) => [Char] ->
                         [(Term.Term (SemanticLabeling.Lab a b) [Char],
                            Term.Term (SemanticLabeling.Lab a b) [Char])]
                           -> Nat.Nat ->
                                ProofChecker.TRSProof a b [Char] ->
                                  Eithera.Eithera [[Char]] ();
checkTRSProof i r m (ProofChecker.DPTrans p prf) =
  ProofChecker.debug i ['D', 'P', ' ', 't', 'r', 'a', 'n', 's']
    (ErrorMonad.bind
      (ErrorMonad.catcha (DpTransImpl.checkDPs r p)
        (\ x ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ',
                 's', 'w', 'i', 't', 'c', 'h', 'i', 'n', 'g', ' ', 'f', 'r',
                 'o', 'm', ' ', 't', 'h', 'e', ' ', 'T', 'R', 'S']
              : List.append (Trs.showTRS r)
                  (['t', 'o', ' ', 't', 'h', 'e', ' ', 'i', 'n', 'i', 't', 'i',
                     'a', 'l', ' ', 'D', 'P', ' ', 'p', 'r', 'o', 'b', 'l', 'e',
                     'm']
                    : List.append (Trs.showDPP p (DpTransImpl.computeID r))
                        x))))
      (\ x ->
        ErrorMonad.catcha
          (ProofChecker.checkDPProof (List.append i ['.', '1'])
            (List.mapa (SemanticLabeling.map_rule SemanticLabeling.init_lab) p)
            (List.mapa (SemanticLabeling.map_rule SemanticLabeling.init_lab)
              (DpTransImpl.computeID r))
            Nat.Zero_nat
            (DpFramework.computeRuleMap
              (List.mapa (SemanticLabeling.map_rule SemanticLabeling.init_lab)
                (DpTransImpl.computeID r)))
            (DpFramework.computeRuleMap
              (DpFramework.reverse_trs
                (List.mapa (SemanticLabeling.map_rule SemanticLabeling.init_lab)
                  (DpTransImpl.computeID r))))
            prf)
          (\ xa ->
            ErrorMonad.errora
              (List.append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 's', 'w', 'i', 't', 'c', 'h', ' ', 't', 'o', ' ',
                   'd', 'e', 'p', 'e', 'n', 'd', 'e', 'n', 'c', 'y', ' ', 'p',
                   'a', 'i', 'r', 's']
                : ProofChecker.indent xa))));
checkTRSProof i r m (ProofChecker.RuleRemoval rp rR prfa) =
  ProofChecker.debug i
    ['r', 'u', 'l', 'e', ' ', 'r', 'e', 'm', 'o', 'v', 'a', 'l']
    (ErrorMonad.bind
      (ErrorMonad.catcha (RedPairImpl.create_rule_removal_proc rp r rR)
        (\ x ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ', 'r', 'u',
                 'l', 'e', ' ', 'r', 'e', 'm', 'o', 'v', 'a', 'l', ' ', 'w',
                 'h', 'e', 'n', ' ', 's', 'w', 'i', 't', 'c', 'h', 'i', 'n',
                 'g', ' ', 'f', 'r', 'o', 'm', ' ', 't', 'h', 'e', ' ', 'T',
                 'R', 'S']
              : List.append (Trs.showTRS r)
                  (['t', 'o', ' ', 't', 'h', 'e', ' ', 'T', 'R', 'S'] :
                    List.append (Trs.showTRS rR) x))))
      (\ x ->
        ErrorMonad.catcha
          (ProofChecker.checkTRSProof (List.append i ['.', '1']) rR m prfa)
          (\ xa ->
            ErrorMonad.errora
              (List.append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 'r', 'u', 'l', 'e', ' ', 'r',
                   'e', 'm', 'o', 'v', 'a', 'l', ' ', 'p', 'r', 'o', 'c', 'e',
                   's', 's', 'o', 'r']
                : ProofChecker.indent xa))));
checkTRSProof i r (Nat.Suc m) (ProofChecker.Unlab uR prfa) =
  ProofChecker.debug i ['u', 'n', 'l', 'a', 'b', 'e', 'l']
    (ErrorMonad.bind
      (ErrorMonad.catcha (SemanticLabeling.unlab_trs_tt r uR)
        (\ x ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ', 'u', 'n',
                 'l', 'a', 'b', 'e', 'l', 'i', 'n', 'g', ' ', 'o', 'f', ' ',
                 'T', 'R', 'S', ' ', 'w', 'h', 'e', 'n', ' ', 's', 'w', 'i',
                 't', 'c', 'h', 'i', 'n', 'g', ' ', 'f', 'r', 'o', 'm', ' ',
                 't', 'h', 'e', ' ', 'T', 'R', 'S']
              : List.append (Trs.showTRS r)
                  (['t', 'o', ' ', 't', 'h', 'e', ' ', 'T', 'R', 'S'] :
                    List.append (Trs.showTRS uR) x))))
      (\ x ->
        ErrorMonad.catcha
          (ProofChecker.checkTRSProof (List.append i ['.', '1']) uR m prfa)
          (\ xa ->
            ErrorMonad.errora
              (List.append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 't', 'h', 'e', ' ', 'u', 'n', 'l', 'a', 'b', 'e',
                   'l', 'i', 'n', 'g']
                : ProofChecker.indent xa))));
checkTRSProof i r Nat.Zero_nat (ProofChecker.Unlab uR prfa) =
  ErrorMonad.errora
    [['u', 'n', 'l', 'a', 'b', 'e', 'l', 'i', 'n', 'g', ' ', 'n', 'o', 't', ' ',
       'a', 'l', 'l', 'o', 'w', 'e', 'd', ' ', 'f', 'o', 'r', ' ', 'l', 'a',
       'b', 'e', 'l', ' ', 'd', 'e', 'p', 't', 'h', ' ', '0']];
checkTRSProof i r m ProofChecker.RisEmpty =
  ProofChecker.debug i ['R', ' ', 'i', 's', ' ', 'e', 'm', 'p', 't', 'y']
    (ErrorMonad.checkB (List.nulla r)
      (List.append
        [List.append i
           [':', ' ', 'R', ' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'e', 'm',
             'p', 't', 'y', ',', ' ', 'b', 'u', 't', ' ', 'i', 't', ' ', 'i',
             's', ' ', 't', 'h', 'e', ' ', 'T', 'R', 'S']]
        (Trs.showTRS r)));
checkTRSProof i r m (ProofChecker.Semlab sli l lR prfa) =
  ProofChecker.debug i ['S', 'e', 'm', 'l', 'a', 'b']
    (ErrorMonad.bind
      (ErrorMonad.catcha (SemanticLabeling.create_sem_lab_tt sli l r lR)
        (\ x ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ',
                 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 's', 'e', 'm',
                 'a', 'n', 't', 'i', 'c', ' ', 'l', 'a', 'b', 'e', 'l', 'i',
                 'n', 'g']
              : x)))
      (\ x ->
        ErrorMonad.catcha
          (ProofChecker.checkTRSProof (List.append i ['.', '1']) lR (Nat.Suc m)
            prfa)
          (\ xa ->
            ErrorMonad.errora
              (List.append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 's', 'e', 'm', 'a', 'n', 't', 'i', 'c', ' ', 'l',
                   'a', 'b', 'e', 'l', 'i', 'n', 'g']
                : ProofChecker.indent xa))));
checkTRSProof i r m (ProofChecker.Rootlab lR prfa) =
  ProofChecker.debug i ['R', 'o', 'o', 't', 'l', 'a', 'b']
    (ErrorMonad.bind
      (ErrorMonad.catcha (RootLabeling.rl_tt r lR)
        (\ x ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ',
                 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 'r', 'o', 'o',
                 't', '-', 'l', 'a', 'b', 'e', 'l', 'i', 'n', 'g']
              : x)))
      (\ x ->
        ErrorMonad.catcha
          (ProofChecker.checkTRSProof (List.append i ['.', '1']) lR (Nat.Suc m)
            prfa)
          (\ xa ->
            ErrorMonad.errora
              (List.append i
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o',
                   'w', ' ', 'r', 'o', 'o', 't', '-', 'l', 'a', 'b', 'e', 'l',
                   'i', 'n', 'g']
                : ProofChecker.indent xa))));

certifyCertProblem ::
  forall a b.
    (Eq a, Show.Sshow a, Eq b,
      Show.Sshow b) => ProofChecker.CertificationProblem a b [Char] ->
                         ProofChecker.CertificationResult;
certifyCertProblem
  (ProofChecker.CertificationProblem True trs (Eithera.Lefta prf)) =
  (case ProofChecker.checkTRSProof ['1']
          (List.mapa (SemanticLabeling.map_rule SemanticLabeling.init_lab) trs)
          Nat.Zero_nat prf
    of {
    Eithera.Lefta err ->
      ProofChecker.Error
        (Show.showLista
          (['t', 'e', 'r', 'm', 'i', 'n', 'a', 't', 'i', 'o', 'n', ' ', 'p',
             'r', 'o', 'o', 'f', ' ', 'n', 'o', 't', ' ', 'a', 'c', 'c', 'e',
             'p', 't', 'e', 'd']
            : err));
    Eithera.Righta x -> ProofChecker.Certified True;
  });
certifyCertProblem
  (ProofChecker.CertificationProblem False trs (Eithera.Righta prfa)) =
  (case ProofChecker.checkTRSDisproof ['1'] trs prfa of {
    Eithera.Lefta err ->
      ProofChecker.Error
        (Show.showLista
          (['n', 'o', 'n', '-', 't', 'e', 'r', 'm', 'i', 'n', 'a', 't', 'i',
             'o', 'n', ' ', 'p', 'r', 'o', 'o', 'f', ' ', 'n', 'o', 't', ' ',
             'a', 'c', 'c', 'e', 'p', 't', 'e', 'd']
            : err));
    Eithera.Righta x -> ProofChecker.Certified False;
  });
certifyCertProblem
  (ProofChecker.CertificationProblem False va (Eithera.Lefta v)) =
  ProofChecker.Error
    ['N', 'o', 't', ' ', 'a', 'l', 'l', 'o', 'w', 'e', 'd', ' ', 't', 'o', ' ',
      'g', 'i', 'v', 'e', ' ', 'a', ' ', 't', 'e', 'r', 'm', 'i', 'n', 'a', 't',
      'i', 'o', 'n', ' ', 'p', 'r', 'o', 'o', 'f', ' ', 'f', 'o', 'r', ' ', 'a',
      ' ', 'n', 'o', 'n', '-', 't', 'e', 'r', 'm', 'i', 'n', 'a', 't', 'i', 'o',
      'n', ' ', 'g', 'o', 'a', 'l', ' ', 'o', 'r', ' ', 'v', 'i', 'c', 'e', '-',
      'v', 'e', 'r', 's', 'a'];
certifyCertProblem
  (ProofChecker.CertificationProblem True va (Eithera.Righta vc)) =
  ProofChecker.Error
    ['N', 'o', 't', ' ', 'a', 'l', 'l', 'o', 'w', 'e', 'd', ' ', 't', 'o', ' ',
      'g', 'i', 'v', 'e', ' ', 'a', ' ', 't', 'e', 'r', 'm', 'i', 'n', 'a', 't',
      'i', 'o', 'n', ' ', 'p', 'r', 'o', 'o', 'f', ' ', 'f', 'o', 'r', ' ', 'a',
      ' ', 'n', 'o', 'n', '-', 't', 'e', 'r', 'm', 'i', 'n', 'a', 't', 'i', 'o',
      'n', ' ', 'g', 'o', 'a', 'l', ' ', 'o', 'r', ' ', 'v', 'i', 'c', 'e', '-',
      'v', 'e', 'r', 's', 'a'];

}
