{-# OPTIONS_GHC -fglasgow-exts #-}

module Term where {

import qualified Nat;
import qualified List;
import qualified Show;
import qualified HOL;
import qualified Product_Type;
import qualified Position;
import qualified Util;

data Term a b = Var b | Fun a [(Term.Term a b)];

newtype Sub a b = Sub (b -> Term.Term a b);

sz :: forall a b. Term.Term a b -> Nat.Nat;
sz (Term.Var x) = Nat.one_nat;
sz (Term.Fun f ts) =
  Nat.plus_nat Nat.one_nat
    (List.foldla (\ a x -> Nat.plus_nat a (Term.sz x)) Nat.Zero_nat ts);

sshow_term ::
  forall c d. (Show.Sshow c, Show.Sshow d) => Term.Term c d -> [Char];
sshow_term (Term.Var x) = Show.sshow x;
sshow_term (Term.Fun f ts) =
  List.append (Show.sshow f)
    (case ts of {
      [] -> Show.sEmpty;
      s : ss ->
        List.append ['(']
          (List.append
            (List.append (Term.sshow_term s)
              (Util.flat_map (\ x -> List.concata [[','], Term.sshow_term x])
                ss))
            [')']);
    });

poss_list :: forall a b. Term.Term a b -> [Position.Pos];
poss_list (Term.Var x) = [Position.Empty];
poss_list (Term.Fun f ts) =
  let {
    a = List.foldra
          (\ t (a @ (i, ps)) ->
            (Nat.minus_nat i Nat.one_nat,
              List.append ps (List.mapa (Position.PCons i) (Term.poss_list t))))
          ts (Nat.minus_nat (List.size_list ts) Nat.one_nat, []);
    (i, aa) = a;
  } in Position.Empty : aa;

data Ctxt a b = Hole
  | More a [(Term.Term a b)] (Term.Ctxt a b) [(Term.Term a b)];

ctxt_of_pos_term :: forall a b. Position.Pos -> Term.Term a b -> Term.Ctxt a b;
ctxt_of_pos_term Position.Empty t = Term.Hole;
ctxt_of_pos_term (Position.PCons i ps) (Term.Fun f ts) =
  Term.More f (List.takea i ts) (Term.ctxt_of_pos_term ps (List.nth ts i))
    (List.dropa (Nat.Suc i) ts);

sub_apply :: forall a b. Term.Term a b -> Term.Sub a b -> Term.Term a b;
sub_apply (Term.Var x) (Term.Sub sigma) = sigma x;
sub_apply (Term.Fun f ss) sigmaa =
  Term.Fun f (List.mapa (\ t -> Term.sub_apply t sigmaa) ss);

ctxt_apply :: forall a b. Term.Ctxt a b -> Term.Term a b -> Term.Term a b;
ctxt_apply Term.Hole s = s;
ctxt_apply (Term.More f ss1 c ss2) s =
  Term.Fun f (List.append ss1 (Term.ctxt_apply c s : ss2));

eq_term :: forall c d. (Eq c, Eq d) => Term.Term c d -> Term.Term c d -> Bool;
eq_term (Term.Fun f ts) (Term.Fun g us) =
  f == g && List.list_all2 Term.eq_term ts us;
eq_term (Term.Var x) (Term.Var y) = x == y;
eq_term (Term.Fun b c) (Term.Var a) = False;
eq_term (Term.Var a) (Term.Fun b c) = False;

instance (Eq a, Eq b) => Eq (Term.Term a b) where {
  a == b = Term.eq_term a b;
};

subt_at :: forall a b. Term.Term a b -> Position.Pos -> Term.Term a b;
subt_at s Position.Empty = s;
subt_at (Term.Fun f ss) (Position.PCons i p) = Term.subt_at (List.nth ss i) p;

sub_case :: forall v f t. ((v -> Term.Term f v) -> t) -> Term.Sub f v -> t;
sub_case f1 (Term.Sub fun) = f1 fun;

args :: forall a b. Term.Term a b -> [(Term.Term a b)];
args (Term.Var uu) = [];
args (Term.Fun uv ts) = ts;

root :: forall a b. Term.Term a b -> Maybe a;
root (Term.Var x) = Nothing;
root (Term.Fun f ts) = Just f;

is_var :: forall f v. Term.Term f v -> Bool;
is_var (Term.Var uu) = True;
is_var (Term.Fun uv uw) = False;

instance (Show.Sshow a, Show.Sshow b) => Show.Sshow (Term.Term a b) where {
  sshow = Term.sshow_term;
};

funs_term_list' :: forall a b. Term.Term a b -> [a];
funs_term_list' (Term.Var uu) = [];
funs_term_list' (Term.Fun f ts) = f : Util.flat_map Term.funs_term_list' ts;

vars_term_list' :: forall a b. Term.Term a b -> [b];
vars_term_list' (Term.Var x) = [x];
vars_term_list' (Term.Fun uu ts) = Util.flat_map Term.vars_term_list' ts;

vars_term_list :: forall a b. (Eq b) => Term.Term a b -> [b];
vars_term_list t = List.remdups (Term.vars_term_list' t);

}
