{-# OPTIONS_GHC -fglasgow-exts #-}

module Xml where {

import qualified Show;
import qualified List;
import qualified Nat;
import qualified Option;
import qualified SubtermCriterionImpl;
import qualified Position;
import qualified Util;
import qualified Term;
import qualified Eithera;
import qualified SemanticLabeling;
import qualified Product_Type;
import qualified RedPairImpl;
import qualified PolyOrder;
import qualified Orderings;
import qualified Integer;
import qualified Matrix;
import qualified ProofChecker;
import qualified Trs;
import qualified ErrorMonad;
import qualified HOL;

data Xml = Xml [Char] [([Char], [Char])] [Xml.Xml] (Maybe [Char]);

data Poly a = PVar Nat.Nat | PNum a | PSum [(Xml.Poly a)]
  | PMult [(Xml.Poly a)];

wspace :: [Char];
wspace = List.append Show.newline (List.append Show.tab [' ']);

trim :: [Char] -> [Char];
trim (c : s) = (if List.member c Xml.wspace then Xml.trim s else c : s);
trim [] = [];

data Domain = Natural | Arctic | Int_mat Nat.Nat | Arc_mat Nat.Nat;

eq_xml :: Xml.Xml -> Xml.Xml -> Bool;
eq_xml (Xml.Xml name1 attrs1 children1 text1)
  (Xml.Xml name2 attrs2 children2 text2) =
  name1 == name2 &&
    (attrs1 == attrs2 &&
      (List.list_all2 Xml.eq_xml children1 children2 && text1 == text2));

instance Eq Xml.Xml where {
  a == b = Xml.eq_xml a b;
};

data Xmldoc = Xmldoc [Char] Xml.Xml;

dquote :: [Char];
dquote = ['\"'];

digit2nat :: Char -> Eithera.Eithera [Char] Nat.Nat;
digit2nat x =
  (if x == '0' then ErrorMonad.returna Nat.Zero_nat
    else (if x == '1' then ErrorMonad.returna Nat.one_nat
           else (if x == '2' then ErrorMonad.returna (Integer.nat 2)
                  else (if x == '3' then ErrorMonad.returna (Integer.nat 3)
                         else (if x == '4'
                                then ErrorMonad.returna (Integer.nat 4)
                                else (if x == '5'
                                       then ErrorMonad.returna (Integer.nat 5)
                                       else (if x == '6'
      then ErrorMonad.returna (Integer.nat 6)
      else (if x == '7' then ErrorMonad.returna (Integer.nat 7)
             else (if x == '8' then ErrorMonad.returna (Integer.nat 8)
                    else (if x == '9' then ErrorMonad.returna (Integer.nat 9)
                           else ErrorMonad.errora
                                  (List.append [x]
                                    [' ', 'i', 's', ' ', 'n', 'o', 't', ' ',
                                      'a', ' ', 'd', 'i', 'g', 'i',
                                      't'])))))))))));

string2natHelp :: Nat.Nat -> [Char] -> Eithera.Eithera [Char] Nat.Nat;
string2natHelp n [] = ErrorMonad.returna n;
string2natHelp n (d : s) =
  ErrorMonad.bind (Xml.digit2nat d)
    (\ m ->
      Xml.string2natHelp (Nat.plus_nat (Nat.times_nat (Integer.nat 10) n) m) s);

string2nat :: [Char] -> Eithera.Eithera [Char] Nat.Nat;
string2nat s =
  (if List.nulla s
    then ErrorMonad.errora
           ['c', 'a', 'n', 'n', 'o', 't', ' ', 'c', 'o', 'n', 'v', 'e', 'r',
             't', ' ', 'e', 'm', 'p', 't', 'y', ' ', 's', 't', 'r', 'i', 'n',
             'g', ' ', 'i', 'n', 't', 'o', ' ', 'n', 'u', 'm', 'b', 'e', 'r']
    else Xml.string2natHelp Nat.Zero_nat s);

showAtt :: ([Char], [Char]) -> [Char];
showAtt (a, v) =
  List.append a
    (List.append ['='] (List.append Xml.dquote (List.append v Xml.dquote)));

showAtts :: [([Char], [Char])] -> [Char];
showAtts asa =
  List.foldra (\ a s -> List.append [' '] (List.append (Xml.showAtt a) s)) asa
    [];

showXMLindent :: [Char] -> Nat.Nat -> Xml.Xml -> [Char];
showXMLindent ind i (Xml.Xml n a c t) =
  List.concata
    [ind, ['<'], n, Xml.showAtts a,
      (if List.nulla c && Option.is_none t then ['/', '>']
        else List.concata
               [['>'],
                 (if not (List.nulla c)
                   then List.append
                          (Util.flat_map
                            (Xml.showXMLindent
                              (List.append ind (List.replicatea i ' ')) i)
                            c)
                          ind
                   else []),
                 (if Option.is_none t then [] else Option.the t), ['<', '/'], n,
                 ['>']])];

sshow_xml :: Xml.Xml -> [Char];
sshow_xml xml = Xml.trim (Xml.showXMLindent Show.newline (Integer.nat 2) xml);

xml2text :: [Char] -> Xml.Xml -> Eithera.Eithera [Char] [Char];
xml2text reqName (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts &&
          (List.nulla cs &&
            (not (Option.is_none text) && not (List.nulla (Option.the text)))))
    then ErrorMonad.returna (Option.the text)
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'e', 'x', 't',
                'r', 'a', 'c', 't', ' ', 't', 'e', 'x', 't', ' ', 'f', 'o', 'r',
                ' '],
               reqName, [' ', 'f', 'r', 'o', 'm', ' ', ' '], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml2nat :: [Char] -> Xml.Xml -> Eithera.Eithera [Char] Nat.Nat;
xml2nat reqName x = ErrorMonad.bind (Xml.xml2text reqName x) Xml.string2nat;

xml3elements ::
  forall a b c d.
    [Char] ->
      (Xml.Xml -> Eithera.Eithera [Char] a) ->
        (Xml.Xml -> Eithera.Eithera [Char] b) ->
          (Xml.Xml -> Eithera.Eithera [Char] c) ->
            (a -> b -> c -> d) -> Xml.Xml -> Eithera.Eithera [Char] d;
xml3elements reqName p1 p2 p3 combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts &&
          (List.size_list cs == Integer.nat 3 && Option.is_none text))
    then ErrorMonad.bind (p1 (List.nth cs Nat.Zero_nat))
           (\ a ->
             ErrorMonad.bind (p2 (List.nth cs Nat.one_nat))
               (\ b ->
                 ErrorMonad.bind (p3 (List.nth cs (Integer.nat 2)))
                   (ErrorMonad.returna . combi a b)))
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

mapMonad ::
  forall a b.
    (Xml.Xml -> Eithera.Eithera a b) -> [Xml.Xml] -> Eithera.Eithera a [b];
mapMonad uu [] = ErrorMonad.returna [];
mapMonad f (xml : rest) =
  (case f xml of {
    Eithera.Lefta a -> ErrorMonad.errora a;
    Eithera.Righta a ->
      (case Xml.mapMonad f rest of {
        Eithera.Lefta aa -> ErrorMonad.errora aa;
        Eithera.Righta asa -> ErrorMonad.returna (a : asa);
      });
  });

xmlManyElements ::
  forall a b.
    [Char] ->
      (Xml.Xml -> Eithera.Eithera [Char] a) ->
        ([a] -> b) -> Xml.Xml -> Eithera.Eithera [Char] b;
xmlManyElements reqName p combi (Xml.Xml name atts cs text) =
  (if name == reqName && (List.nulla atts && Option.is_none text)
    then ErrorMonad.bind (Xml.mapMonad p cs) (ErrorMonad.returna . combi)
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml2sp ::
  forall a.
    (Xml.Xml -> Eithera.Eithera [Char] a) ->
      Xml.Xml -> Eithera.Eithera [Char] (SubtermCriterionImpl.SpL a);
xml2sp xml2name =
  Xml.xmlManyElements
    ['a', 'r', 'g', 'u', 'm', 'e', 'n', 't', 'F', 'i', 'l', 't', 'e', 'r']
    (Xml.xml3elements
      ['a', 'r', 'g', 'u', 'm', 'e', 'n', 't', 'F', 'i', 'l', 't', 'e', 'r',
        'E', 'n', 't', 'r', 'y']
      xml2name (Xml.xml2nat ['a', 'r', 'i', 't', 'y'])
      (Xml.xml2nat ['c', 'o', 'l', 'l', 'a', 'p', 's', 'i', 'n', 'g'])
      (\ f a n -> (f, Nat.minus_nat n Nat.one_nat)))
    SubtermCriterionImpl.SimpProj;

parseError :: forall a. [Char] -> [Char] -> Eithera.Eithera [Char] a;
parseError s allowed =
  ErrorMonad.errora
    (List.append
      ['e', 'x', 'p', 'e', 'c', 't', 'e', 'd', ' ', 's', 'y', 'm', 'b', 'o',
        'l', 's', ' ']
      (List.append allowed
        (List.append [' ', 'b', 'u', 't', ' ', 'f', 'o', 'u', 'n', 'd', ' ']
          (List.takea (Integer.nat 30) s))));

consume2 ::
  [[Char]] -> [[Char]] -> [Char] -> Eithera.Eithera [Char] ([Char], [Char]);
consume2 [] allowed s =
  Xml.parseError s (Util.flat_map (\ sa -> List.append sa [',']) allowed);
consume2 (t : asa) allowed s =
  (if List.mapa snd (List.zipa t s) == t
    then ErrorMonad.returna (t, Xml.trim (List.dropa (List.size_list t) s))
    else Xml.consume2 asa allowed s);

consume :: [[Char]] -> [Char] -> Eithera.Eithera [Char] ([Char], [Char]);
consume allowed s = Xml.consume2 allowed allowed s;

letters :: [Char];
letters =
  ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
    'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B', 'C', 'D',
    'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S',
    'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '_', '0', '1', '2', '3', '4', '5', '6',
    '7', '8', '9', '&', ';', ':'];

nat2int :: Nat.Nat -> Integer;
nat2int Nat.Zero_nat = 0;
nat2int (Nat.Suc n) = 1 + Xml.nat2int n;

string2int :: [Char] -> Eithera.Eithera [Char] Integer;
string2int s =
  (case s of {
    [] -> ErrorMonad.errora
            ['c', 'a', 'n', 'n', 'o', 't', ' ', 'c', 'o', 'n', 'v', 'e', 'r',
              't', ' ', 'e', 'm', 'p', 't', 'y', ' ', 's', 't', 'r', 'i', 'n',
              'g', ' ', 'i', 'n', 't', 'o', ' ', 'n', 'u', 'm', 'b', 'e', 'r'];
    h : t ->
      let {
        a = (if h == '-' then (False, t) else (True, s));
        (pos, ss) = a;
      } in ErrorMonad.bind (Xml.string2nat ss)
             (\ n ->
               (if pos || n == Nat.Zero_nat
                 then ErrorMonad.returna (Xml.nat2int n)
                 else ErrorMonad.returna (0 - Xml.nat2int n)));
  });

xml2int :: [Char] -> Xml.Xml -> Eithera.Eithera [Char] Integer;
xml2int reqName x = ErrorMonad.bind (Xml.xml2text reqName x) Xml.string2int;

list2position :: [Nat.Nat] -> Position.Pos;
list2position [] = Position.Empty;
list2position (n : ns) = Position.PCons n (Xml.list2position ns);

xmlChange ::
  forall a b.
    (Xml.Xml -> Eithera.Eithera [Char] a) ->
      (a -> b) -> Xml.Xml -> Eithera.Eithera [Char] b;
xmlChange p f x = ErrorMonad.bind (p x) (\ a -> ErrorMonad.returna (f a));

xml2position :: Xml.Xml -> Eithera.Eithera [Char] Nat.Nat;
xml2position =
  Xml.xmlChange (Xml.xml2nat ['p', 'o', 's', 'i', 't', 'i', 'o', 'n'])
    (\ n -> Nat.minus_nat n Nat.one_nat);

xml2pos :: Xml.Xml -> Eithera.Eithera [Char] Position.Pos;
xml2pos =
  Xml.xmlManyElements
    ['p', 'o', 's', 'i', 't', 'i', 'o', 'n', 'I', 'n', 'T', 'e', 'r', 'm']
    Xml.xml2position Xml.list2position;

string2bool :: [Char] -> Eithera.Eithera [Char] Bool;
string2bool s =
  (if s == ['t', 'r', 'u', 'e'] then ErrorMonad.returna True
    else (if s == ['f', 'a', 'l', 's', 'e'] then ErrorMonad.returna False
           else ErrorMonad.errora
                  (List.append
                    (List.append
                      ['c', 'a', 'n', 'n', 'o', 't', ' ', 'c', 'o', 'n', 'v',
                        'e', 'r', 't', ' ']
                      s)
                    [' ', 'i', 'n', 't', 'o', ' ', 'B', 'o', 'o', 'l', 'e', 'a',
                      'n'])));

xml2bool :: [Char] -> Xml.Xml -> Eithera.Eithera [Char] Bool;
xml2bool reqName x = ErrorMonad.bind (Xml.xml2text reqName x) Xml.string2bool;

xml1element ::
  forall a b.
    [Char] ->
      (Xml.Xml -> Eithera.Eithera [Char] a) ->
        (a -> b) -> Xml.Xml -> Eithera.Eithera [Char] b;
xml1element reqName p1 combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts &&
          (List.size_list cs == Nat.one_nat && Option.is_none text))
    then ErrorMonad.bind (p1 (List.nth cs Nat.Zero_nat))
           (ErrorMonad.returna . combi)
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml1CmanyElements ::
  forall a b c.
    [Char] ->
      (Xml.Xml -> Eithera.Eithera [Char] a) ->
        (a -> Xml.Xml -> Eithera.Eithera [Char] b) ->
          (a -> [b] -> c) -> Xml.Xml -> Eithera.Eithera [Char] c;
xml1CmanyElements reqName p1 p2 combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts && (Option.is_none text && not (List.nulla cs)))
    then ErrorMonad.bind (p1 (List.hd cs))
           (\ a ->
             ErrorMonad.bind (Xml.mapMonad (p2 a) (List.tl cs))
               (ErrorMonad.returna . combi a))
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml1manyElements ::
  forall a b c.
    [Char] ->
      (Xml.Xml -> Eithera.Eithera [Char] a) ->
        (Xml.Xml -> Eithera.Eithera [Char] b) ->
          (a -> [b] -> c) -> Xml.Xml -> Eithera.Eithera [Char] c;
xml1manyElements reqName p1 p2 = Xml.xml1CmanyElements reqName p1 (\ a -> p2);

xml2var :: forall a. Xml.Xml -> Eithera.Eithera [Char] (Term.Term a [Char]);
xml2var x =
  ErrorMonad.bind (Xml.xml2text ['v', 'a', 'r'] x)
    (ErrorMonad.returna . Term.Var);

xmlGetName :: Xml.Xml -> [Char];
xmlGetName (Xml.Xml name uu uv uw) = name;

xmlOptions ::
  forall a.
    [([Char], Xml.Xml -> Eithera.Eithera [Char] a)] ->
      Xml.Xml -> Eithera.Eithera [Char] a;
xmlOptions ps x =
  (case Util.lookupa (Xml.xmlGetName x) ps of {
    Nothing ->
      ErrorMonad.errora
        (List.concata
          [['e', 'x', 'p', 'e', 'c', 't', 'e', 'd', ' ', 'o', 'n', 'e', ' ',
             'o', 'f', ':', ' '],
            Util.flat_map (\ p -> List.append (fst p) [' ']) ps, Show.newline,
            ['b', 'u', 't', ' ', 'f', 'o', 'u', 'n', 'd'], Show.newline,
            Xml.sshow_xml x]);
    Just p -> p x;
  });

xml2term ::
  forall a.
    (Xml.Xml -> Eithera.Eithera [Char] a) ->
      Xml.Xml -> Eithera.Eithera [Char] (Term.Term a [Char]);
xml2term xml2name x =
  Xml.xmlOptions
    [(['v', 'a', 'r'], Xml.xml2var),
      (['f', 'u', 'n', 'a', 'p', 'p'],
        Xml.xml1manyElements ['f', 'u', 'n', 'a', 'p', 'p'] xml2name
          (Xml.xml1element ['a', 'r', 'g'] (Xml.xml2term xml2name) id)
          Term.Fun)]
    x;

xml2elements ::
  forall a b c.
    [Char] ->
      (Xml.Xml -> Eithera.Eithera [Char] a) ->
        (Xml.Xml -> Eithera.Eithera [Char] b) ->
          (a -> b -> c) -> Xml.Xml -> Eithera.Eithera [Char] c;
xml2elements reqName p1 p2 combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts &&
          (List.size_list cs == Integer.nat 2 && Option.is_none text))
    then ErrorMonad.bind (p1 (List.nth cs Nat.Zero_nat))
           (\ a ->
             ErrorMonad.bind (p2 (List.nth cs Nat.one_nat))
               (ErrorMonad.returna . combi a))
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml2rule ::
  forall a.
    (Xml.Xml -> Eithera.Eithera [Char] a) ->
      Xml.Xml ->
        Eithera.Eithera [Char] (Term.Term a [Char], Term.Term a [Char]);
xml2rule xml2name =
  Xml.xml2elements ['r', 'u', 'l', 'e']
    (Xml.xml1element ['l', 'h', 's'] (Xml.xml2term xml2name) id)
    (Xml.xml1element ['r', 'h', 's'] (Xml.xml2term xml2name) id)
    (\ a b -> (a, b));

xml2scg ::
  forall f.
    (Xml.Xml -> Eithera.Eithera [Char] f) ->
      Xml.Xml ->
        Eithera.Eithera [Char]
          ((Term.Term f [Char], Term.Term f [Char]),
            ([(Nat.Nat, Nat.Nat)], [(Nat.Nat, Nat.Nat)]));
xml2scg xml2name =
  Xml.xml1manyElements
    ['s', 'i', 'z', 'e', 'C', 'h', 'a', 'n', 'g', 'e', 'G', 'r', 'a', 'p', 'h']
    (Xml.xml2rule xml2name)
    (Xml.xml3elements ['e', 'd', 'g', 'e'] Xml.xml2position
      (Xml.xml2bool ['s', 't', 'r', 'i', 'c', 't']) Xml.xml2position
      (\ p s q -> (s, (p, q))))
    (\ lr edges ->
      (lr, (List.mapa snd (List.filtera fst edges),
             List.mapa snd (List.filtera (\ x -> not (fst x)) edges))));

splitterFun :: (Char -> Bool) -> [Char] -> ([Char], [Char]);
splitterFun allowed (c : s) =
  (if allowed c
    then let {
           a = Xml.splitterFun allowed s;
           (l, aa) = a;
         } in (c : l, aa)
    else ([], c : s));
splitterFun uu [] = ([], []);

parseText :: [Char] -> Eithera.Eithera [Char] (Maybe [Char], [Char]);
parseText s =
  let {
    a = Xml.splitterFun (\ c -> not ([c] == ['<'])) s;
    (l, r) = a;
  } in (if List.nulla (Xml.trim l) then ErrorMonad.returna (Nothing, r)
         else ErrorMonad.returna
                (Just (List.rev (Xml.trim (List.rev (Xml.trim l)))), r));

parseAttribute :: [Char] -> Eithera.Eithera [Char] ([Char], [Char]);
parseAttribute s =
  ErrorMonad.bind (Xml.consume [Xml.dquote] s)
    (\ (a @ (uu, s1)) ->
      let {
        aa = Xml.splitterFun (\ c -> not ([c] == Xml.dquote)) s1;
        (n, s2) = aa;
      } in ErrorMonad.bind (Xml.consume [Xml.dquote] s2)
             (\ (ab @ (uua, s3)) -> ErrorMonad.returna (n, s3)));

splitter :: [Char] -> [Char] -> ([Char], [Char]);
splitter allowed s = Xml.splitterFun (\ c -> List.member c allowed) s;

parseName :: [Char] -> Eithera.Eithera [Char] ([Char], [Char]);
parseName s =
  let {
    a = Xml.splitter Xml.letters s;
    (l, r) = a;
  } in (if List.nulla l then Xml.parseError s Xml.letters
         else ErrorMonad.returna (l, Xml.trim r));

parseAttributes ::
  [Char] -> Eithera.Eithera [Char] ([([Char], [Char])], [Char]);
parseAttributes [] = ErrorMonad.returna ([], []);
parseAttributes (c : s) =
  (if List.member c ['/', '>'] then ErrorMonad.returna ([], c : s)
    else ErrorMonad.bind (Xml.parseName (c : s))
           (\ (a @ (n, s1)) ->
             ErrorMonad.bind (Xml.consume [['=']] s1)
               (\ (aa @ (uu, s2)) ->
                 ErrorMonad.bind (Xml.parseAttribute s2)
                   (\ (ab @ (ac, s3)) ->
                     ErrorMonad.bind (Xml.parseAttributes s3)
                       (\ (b @ (nas, s4)) ->
                         ErrorMonad.returna ((n, ac) : nas, s4))))));

parseXMLList :: [Char] -> Eithera.Eithera [Char] ([Xml.Xml], [Char]);
parseXMLList s =
  (if List.nulla s || List.takea (Integer.nat 2) s == ['<', '/']
    then ErrorMonad.returna ([], s)
    else ErrorMonad.bind (Xml.consume [['<']] s)
           (\ (a @ (uu, s1)) ->
             ErrorMonad.bind (Xml.parseName s1)
               (\ (aa @ (n, s2)) ->
                 ErrorMonad.bind (Xml.parseAttributes s2)
                   (\ (ab @ (nas, s3)) ->
                     ErrorMonad.bind (Xml.consume [['/', '>'], ['>']] s3)
                       (\ (ac @ (clo, s4)) ->
                         (if clo == ['/', '>']
                           then ErrorMonad.bind (Xml.parseXMLList s4)
                                  (\ (ad @ (li, s5)) ->
                                    ErrorMonad.returna
                                      (Xml.Xml n nas [] Nothing : li, s5))
                           else ErrorMonad.bind (Xml.parseText s4)
                                  (\ (ad @ (t, s5)) ->
                                    ErrorMonad.bind (Xml.parseXMLList s5)
                                      (\ (ae @ (chi, s6)) ->
ErrorMonad.bind (Xml.consume [List.append ['<', '/'] (List.append n ['>'])] s6)
  (\ (af @ (uua, s7)) ->
    ErrorMonad.bind (Xml.parseXMLList s7)
      (\ (ag @ (li, s8)) ->
        ErrorMonad.returna (Xml.Xml n nas chi t : li, s8)))))))))));

parseXML :: [Char] -> Eithera.Eithera [Char] (Xml.Xml, [Char]);
parseXML s =
  ErrorMonad.bind (Xml.consume [['<']] s)
    (\ (a @ (uu, s1)) ->
      ErrorMonad.bind (Xml.parseName s1)
        (\ (aa @ (n, s2)) ->
          ErrorMonad.bind (Xml.parseAttributes s2)
            (\ (ab @ (nas, s3)) ->
              ErrorMonad.bind (Xml.consume [['/', '>'], ['>']] s3)
                (\ (ac @ (clo, s4)) ->
                  (if clo == ['/', '>']
                    then ErrorMonad.returna
                           (Xml.Xml n nas [] Nothing, Xml.trim s4)
                    else ErrorMonad.bind (Xml.parseText s4)
                           (\ (ad @ (t, s5)) ->
                             ErrorMonad.bind (Xml.parseXMLList s5)
                               (\ (ae @ (chi, s6)) ->
                                 ErrorMonad.bind
                                   (Xml.consume
                                     [List.append ['<', '/']
(List.append n ['>'])]
                                     s6)
                                   (\ (af @ (uua, s7)) ->
                                     ErrorMonad.returna
                                       (Xml.Xml n nas chi t, s7)))))))));

scanUpto :: [Char] -> [Char] -> Eithera.Eithera [Char] ([Char], [Char]);
scanUpto end [] =
  ErrorMonad.errora
    (List.append
      ['d', 'i', 'd', ' ', 'n', 'o', 't', ' ', 'f', 'i', 'n', 'd', ' ', 'r',
        'e', 'q', 'u', 'i', 'r', 'e', 'd', ' ', 't', 'o', 'k', 'e', 'n', ' ']
      end);
scanUpto end (c : s) =
  (if List.mapa snd (List.zipa end (c : s)) == end
    then ErrorMonad.returna
           (List.takea (List.size_list end) (c : s),
             Xml.trim (List.dropa (List.size_list end) (c : s)))
    else ErrorMonad.bind (Xml.scanUpto end s)
           (\ (a @ (res, rest)) -> ErrorMonad.returna (c : res, rest)));

xml2poly ::
  forall a.
    (Xml.Xml -> Eithera.Eithera [Char] a) ->
      Xml.Xml -> Eithera.Eithera [Char] (Xml.Poly a);
xml2poly xml2coeff x =
  Xml.xml1element ['p', 'o', 'l', 'y', 'n', 'o', 'm', 'i', 'a', 'l']
    (Xml.xmlOptions
      [(['c', 'o', 'e', 'f', 'f', 'i', 'c', 'i', 'e', 'n', 't'],
         Xml.xml1element ['c', 'o', 'e', 'f', 'f', 'i', 'c', 'i', 'e', 'n', 't']
           xml2coeff Xml.PNum),
        (['v', 'a', 'r', 'i', 'a', 'b', 'l', 'e'],
          Xml.xmlChange (Xml.xml2nat ['v', 'a', 'r', 'i', 'a', 'b', 'l', 'e'])
            (\ n -> Xml.PVar (Nat.minus_nat n Nat.one_nat))),
        (['s', 'u', 'm'],
          Xml.xmlManyElements ['s', 'u', 'm'] (Xml.xml2poly xml2coeff)
            Xml.PSum),
        (['p', 'r', 'o', 'd', 'u', 'c', 't'],
          Xml.xmlManyElements ['p', 'r', 'o', 'd', 'u', 'c', 't']
            (Xml.xml2poly xml2coeff) Xml.PMult)])
    id x;

xml_term ::
  forall c d. (Show.Sshow c, Show.Sshow d) => Term.Term c d -> Xml.Xml;
xml_term (Term.Var x) = Xml.Xml ['v', 'a', 'r'] [] [] (Just (Show.sshow x));
xml_term (Term.Fun f ts) =
  Xml.Xml ['f', 'u', 'n', 'a', 'p', 'p'] []
    (Xml.Xml ['n', 'a', 'm', 'e'] [] [] (Just (Show.sshow f)) :
      List.mapa (\ t -> Xml.Xml ['a', 'r', 'g'] [] [Xml.xml_term t] Nothing) ts)
    Nothing;

fitLength :: forall a. a -> Nat.Nat -> [a] -> [a];
fitLength uu Nat.Zero_nat uv = [];
fitLength a (Nat.Suc v) [] = List.replicatea (Nat.Suc v) a;
fitLength a (Nat.Suc n) (b : asa) = b : Xml.fitLength a n asa;

sum_lpoly :: forall a. (a -> a -> a) -> [a] -> [a] -> [a];
sum_lpoly uu [] ys = ys;
sum_lpoly uv (v : va) [] = v : va;
sum_lpoly pl (x : xs) (y : ys) = pl x y : Xml.sum_lpoly pl xs ys;

xml2label ::
  forall a.
    (Xml.Xml -> Eithera.Eithera [Char] a) ->
      Xml.Xml -> Eithera.Eithera [Char] (Eithera.Eithera [Nat.Nat] [a]);
xml2label xml2name xml =
  ErrorMonad.bind
    (Xml.xmlManyElements ['l', 'a', 'b', 'e', 'l']
      (Xml.xmlOptions
        [(['n', 'u', 'm', 'b', 'e', 'r'],
           Xml.xmlChange (Xml.xml2nat ['n', 'u', 'm', 'b', 'e', 'r'])
             Eithera.Lefta),
          (['n', 'a', 'm', 'e'], Xml.xmlChange xml2name Eithera.Righta),
          (['s', 'h', 'a', 'r', 'p'], Xml.xmlChange xml2name Eithera.Righta),
          (['l', 'a', 'b', 'e', 'l', 'e', 'd', 'S', 'y', 'm', 'b', 'o', 'l'],
            Xml.xmlChange xml2name Eithera.Righta)])
      id xml)
    (\ xs ->
      (if Util.foralla Eithera.isLeft xs
        then ErrorMonad.returna (Eithera.Lefta (List.mapa Eithera.lefta xs))
        else (if Util.foralla Eithera.isRight xs
               then ErrorMonad.returna
                      (Eithera.Righta (List.mapa Eithera.righta xs))
               else ErrorMonad.errora
                      ['m', 'i', 'x', 'e', 'd', ' ', 'l', 'a', 'b', 'e', 'l',
                        's'])));

xml2rules ::
  forall a.
    (Xml.Xml -> Eithera.Eithera [Char] a) ->
      Xml.Xml ->
        Eithera.Eithera [Char] [(Term.Term a [Char], Term.Term a [Char])];
xml2rules xml2name =
  Xml.xmlManyElements ['r', 'u', 'l', 'e', 's'] (Xml.xml2rule xml2name) id;

xml_rules ::
  forall a c.
    (Show.Sshow a, Show.Sshow c) => [(Term.Term a c, Term.Term a c)] -> Xml.Xml;
xml_rules rls =
  Xml.Xml ['r', 'u', 'l', 'e', 's'] []
    (List.mapa
      (\ (a @ (l, r)) ->
        Xml.Xml ['r', 'u', 'l', 'e'] []
          [Xml.Xml ['l', 'h', 's'] [] [Xml.xml_term l] Nothing,
            Xml.Xml ['r', 'h', 's'] [] [Xml.xml_term r] Nothing]
          Nothing)
      rls)
    Nothing;

domain_case ::
  forall t. t -> t -> (Nat.Nat -> t) -> (Nat.Nat -> t) -> Xml.Domain -> t;
domain_case f1 f2 f3 f4 (Xml.Arc_mat nat) = f4 nat;
domain_case f1 f2 f3 f4 (Xml.Int_mat nat) = f3 nat;
domain_case f1 f2 f3 f4 Xml.Arctic = f2;
domain_case f1 f2 f3 f4 Xml.Natural = f1;

id_labeling ::
  forall f a.
    SemanticLabeling.Lab f [a] ->
      [a] -> Eithera.Eithera [a] [(SemanticLabeling.Lab f [a])];
id_labeling uu cs = Eithera.Lefta cs;

stripXMLHeader :: [Char] -> Eithera.Eithera [Char] ([Char], [Char]);
stripXMLHeader s =
  (if List.takea (Integer.nat 2) (Xml.trim s) == ['<', '?']
    then ErrorMonad.bind (Xml.scanUpto ['?', '>'] s)
           (\ (a @ (header1, s1)) ->
             ErrorMonad.bind (Xml.stripXMLHeader s1)
               (\ (aa @ (header2, s2)) ->
                 ErrorMonad.returna (List.append header1 header2, Xml.trim s2)))
    else ErrorMonad.returna ([], Xml.trim s));

parseXMLDoc :: [Char] -> Eithera.Eithera [Char] Xml.Xmldoc;
parseXMLDoc s =
  ProofChecker.debug ['0']
    ['p', 'a', 'r', 's', 'i', 'n', 'g', ' ', 't', 'x', 't', ' ', 't', 'o', ' ',
      'x', 'm', 'l']
    (ErrorMonad.bind (Xml.stripXMLHeader s)
      (\ (a @ (header, rest)) ->
        ErrorMonad.bind (Xml.parseXML rest)
          (\ (aa @ (xml, end)) ->
            (if List.nulla end then ErrorMonad.returna (Xml.Xmldoc header xml)
              else ErrorMonad.errora
                     (List.append
                       ['t', 'h', 'e', 'r', 'e', ' ', 'i', 's', ' ', 't', 'e',
                         'x', 't', ' ', 'a', 'f', 't', 'e', 'r', ' ', 'r', 'o',
                         'o', 't', ' ', 'e', 'l', 'e', 'm', 'e', 'n', 't', ':',
                         ' ']
                       (List.append Show.newline end))))));

xml4elements ::
  forall a b c d e.
    [Char] ->
      (Xml.Xml -> Eithera.Eithera [Char] a) ->
        (Xml.Xml -> Eithera.Eithera [Char] b) ->
          (Xml.Xml -> Eithera.Eithera [Char] c) ->
            (Xml.Xml -> Eithera.Eithera [Char] d) ->
              (a -> b -> c -> d -> e) -> Xml.Xml -> Eithera.Eithera [Char] e;
xml4elements reqName p1 p2 p3 p4 combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts &&
          (List.size_list cs == Integer.nat 4 && Option.is_none text))
    then ErrorMonad.bind (p1 (List.nth cs Nat.Zero_nat))
           (\ a ->
             ErrorMonad.bind (p2 (List.nth cs Nat.one_nat))
               (\ b ->
                 ErrorMonad.bind (p3 (List.nth cs (Integer.nat 2)))
                   (\ c ->
                     ErrorMonad.bind (p4 (List.nth cs (Integer.nat 3)))
                       (ErrorMonad.returna . combi a b c))))
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml0elements :: forall a. [Char] -> a -> Xml.Xml -> Eithera.Eithera [Char] a;
xml0elements reqName combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts && (List.nulla cs && Option.is_none text))
    then ErrorMonad.returna combi
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml2context ::
  forall a.
    (Xml.Xml -> Eithera.Eithera [Char] a) ->
      Xml.Xml -> Eithera.Eithera [Char] (Term.Ctxt a [Char]);
xml2context xml2name x =
  Xml.xmlOptions
    [(['b', 'o', 'x'], Xml.xml0elements ['b', 'o', 'x'] Term.Hole),
      (['f', 'u', 'n', 'C', 'o', 'n', 't', 'e', 'x', 't'],
        Xml.xml4elements ['f', 'u', 'n', 'C', 'o', 'n', 't', 'e', 'x', 't']
          xml2name
          (Xml.xmlManyElements ['b', 'e', 'f', 'o', 'r', 'e']
            (Xml.xml2term xml2name) id)
          (Xml.xml2context xml2name)
          (Xml.xmlManyElements ['a', 'f', 't', 'e', 'r'] (Xml.xml2term xml2name)
            id)
          Term.More)]
    x;

xml2arith_fun :: Xml.Xml -> Eithera.Eithera [Char] SemanticLabeling.ArithFun;
xml2arith_fun xml =
  Xml.xmlOptions
    [(['n', 'a', 't', 'u', 'r', 'a', 'l'],
       Xml.xmlChange (Xml.xml2nat ['n', 'a', 't', 'u', 'r', 'a', 'l'])
         SemanticLabeling.Const),
      (['v', 'a', 'r', 'i', 'a', 'b', 'l', 'e'],
        Xml.xmlChange (Xml.xml2nat ['v', 'a', 'r', 'i', 'a', 'b', 'l', 'e'])
          (\ n -> SemanticLabeling.Arg (Nat.minus_nat n Nat.one_nat))),
      (['s', 'u', 'm'],
        Xml.xmlManyElements ['s', 'u', 'm'] Xml.xml2arith_fun
          SemanticLabeling.Sum),
      (['p', 'r', 'o', 'd', 'u', 'c', 't'],
        Xml.xmlManyElements ['p', 'r', 'o', 'd', 'u', 'c', 't']
          Xml.xml2arith_fun SemanticLabeling.Prod),
      (['m', 'i', 'n', 'i', 'm', 'u', 'm'],
        Xml.xmlManyElements ['m', 'i', 'n', 'i', 'm', 'u', 'm']
          Xml.xml2arith_fun SemanticLabeling.Min),
      (['m', 'a', 'x', 'i', 'm', 'u', 'm'],
        Xml.xmlManyElements ['m', 'a', 'x', 'i', 'm', 'u', 'm']
          Xml.xml2arith_fun SemanticLabeling.Max)]
    xml;

xml2sl_inter ::
  forall f l.
    (Xml.Xml -> Eithera.Eithera [Char] (SemanticLabeling.Lab f l)) ->
      Xml.Xml -> Eithera.Eithera [Char] (Maybe (SemanticLabeling.Sl_inter f l));
xml2sl_inter xml2name =
  Xml.xml1element ['m', 'o', 'd', 'e', 'l']
    (Xml.xmlOptions
      [(['f', 'i', 'n', 'i', 't', 'e', 'M', 'o', 'd', 'e', 'l'],
         Xml.xml1manyElements
           ['f', 'i', 'n', 'i', 't', 'e', 'M', 'o', 'd', 'e', 'l']
           (Xml.xmlChange
             (Xml.xml2nat
               ['c', 'a', 'r', 'r', 'i', 'e', 'r', 'S', 'i', 'z', 'e'])
             (\ n -> Nat.minus_nat n Nat.one_nat))
           (Xml.xml3elements ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't']
             xml2name (Xml.xml2nat ['a', 'r', 'i', 't', 'y']) Xml.xml2arith_fun
             (\ f a -> (\ b -> ((f, a), b))))
           (\ n xs -> Just (SemanticLabeling.SL_Inter n xs))),
        (['r', 'o', 'o', 't', 'L', 'a', 'b', 'e', 'l', 'i', 'n', 'g'],
          Xml.xml0elements
            ['r', 'o', 'o', 't', 'L', 'a', 'b', 'e', 'l', 'i', 'n', 'g']
            Nothing)])
    id;

xml5elements ::
  forall a b c d e f.
    [Char] ->
      (Xml.Xml -> Eithera.Eithera [Char] a) ->
        (Xml.Xml -> Eithera.Eithera [Char] b) ->
          (Xml.Xml -> Eithera.Eithera [Char] c) ->
            (Xml.Xml -> Eithera.Eithera [Char] d) ->
              (Xml.Xml -> Eithera.Eithera [Char] e) ->
                (a -> b -> c -> d -> e -> f) ->
                  Xml.Xml -> Eithera.Eithera [Char] f;
xml5elements reqName p1 p2 p3 p4 p5 combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts &&
          (List.size_list cs == Integer.nat 5 && Option.is_none text))
    then ErrorMonad.bind (p1 (List.nth cs Nat.Zero_nat))
           (\ a ->
             ErrorMonad.bind (p2 (List.nth cs Nat.one_nat))
               (\ b ->
                 ErrorMonad.bind (p3 (List.nth cs (Integer.nat 2)))
                   (\ c ->
                     ErrorMonad.bind (p4 (List.nth cs (Integer.nat 3)))
                       (\ d ->
                         ErrorMonad.bind (p5 (List.nth cs (Integer.nat 4)))
                           (ErrorMonad.returna . combi a b c d)))))
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml2vec_coeff ::
  forall a.
    (Xml.Xml -> Eithera.Eithera [Char] a) ->
      Xml.Xml -> Eithera.Eithera [Char] [a];
xml2vec_coeff xml2coeff =
  Xml.xmlManyElements ['v', 'e', 'c', 't', 'o', 'r']
    (Xml.xml1element ['c', 'o', 'e', 'f', 'f', 'i', 'c', 'i', 'e', 'n', 't']
      xml2coeff id)
    id;

xml2mat_coeff ::
  forall a.
    (Xml.Xml -> Eithera.Eithera [Char] a) ->
      Xml.Xml -> Eithera.Eithera [Char] [[a]];
xml2mat_coeff xml2coeff =
  Xml.xmlManyElements ['m', 'a', 't', 'r', 'i', 'x']
    (Xml.xml2vec_coeff xml2coeff) id;

xml2arctic_coeff :: Xml.Xml -> Eithera.Eithera [Char] RedPairImpl.Arctic;
xml2arctic_coeff =
  Xml.xmlOptions
    [(['i', 'n', 't', 'e', 'g', 'e', 'r'],
       Xml.xmlChange (Xml.xml2int ['i', 'n', 't', 'e', 'g', 'e', 'r'])
         RedPairImpl.Num_arc),
      (['m', 'i', 'n', 'u', 's', 'I', 'n', 'f', 'i', 'n', 'i', 't', 'y'],
        Xml.xml0elements
          ['m', 'i', 'n', 'u', 's', 'I', 'n', 'f', 'i', 'n', 'i', 't', 'y']
          RedPairImpl.MinInfty)];

poly_to_lpoly ::
  forall a.
    (Eq a) => a -> a -> (a -> a -> a) ->
                          (a -> a -> a) ->
                            Xml.Poly a -> Eithera.Eithera [Char] (a, [a]);
poly_to_lpoly uu uv uw ux (Xml.PNum i) = ErrorMonad.returna (i, []);
poly_to_lpoly ze on pl ti (Xml.PVar x) =
  ErrorMonad.returna (ze, List.append (List.replicatea x ze) [on]);
poly_to_lpoly ze on pl ti (Xml.PSum [p]) = Xml.poly_to_lpoly ze on pl ti p;
poly_to_lpoly ze on pl ti (Xml.PSum (p : q : ps)) =
  ErrorMonad.bind (Xml.poly_to_lpoly ze on pl ti p)
    (\ (a @ (cp, ncp)) ->
      ErrorMonad.bind (Xml.poly_to_lpoly ze on pl ti (Xml.PSum (q : ps)))
        (\ (aa @ (cq, ncq)) ->
          ErrorMonad.returna (pl cp cq, Xml.sum_lpoly pl ncp ncq)));
poly_to_lpoly ze on pl ti (Xml.PMult [p]) = Xml.poly_to_lpoly ze on pl ti p;
poly_to_lpoly ze on pl ti (Xml.PMult (p : q : ps)) =
  ErrorMonad.bind (Xml.poly_to_lpoly ze on pl ti p)
    (\ (a @ (cp, ncp)) ->
      ErrorMonad.bind (Xml.poly_to_lpoly ze on pl ti (Xml.PMult (q : ps)))
        (\ (aa @ (cq, ncq)) ->
          (if Util.foralla (\ ab -> ze == ab) ncp
            then ErrorMonad.returna (ti cp cq, List.mapa (ti cp) ncq)
            else (if Util.foralla (\ ab -> ze == ab) ncq
                   then ErrorMonad.returna
                          (ti cp cq, List.mapa (\ x -> ti x cq) ncp)
                   else ErrorMonad.errora
                          ['n', 'o', 'n', '-', 'l', 'i', 'n', 'e', 'a', 'r',
                            ' ', 'p', 'o', 'l', 'y', 'n', 'o', 'm', 'i', 'a',
                            'l', 's', ' ', 'n', 'o', 't', ' ', 's', 'u', 'p',
                            'p', 'o', 'r', 't', 'e', 'd']))));

xml2int_coeff :: Xml.Xml -> Eithera.Eithera [Char] Integer;
xml2int_coeff = Xml.xml2int ['i', 'n', 't', 'e', 'g', 'e', 'r'];

xml2redpair ::
  forall a.
    (Xml.Xml -> Eithera.Eithera [Char] a) ->
      Xml.Xml -> Eithera.Eithera [Char] (RedPairImpl.Redpair a);
xml2redpair xml2name =
  Xml.xmlOptions
    [(['r', 'e', 'd', 'P', 'a', 'i', 'r'],
       Xml.xml1element ['r', 'e', 'd', 'P', 'a', 'i', 'r']
         (Xml.xml1CmanyElements
           ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't', 'a', 't', 'i', 'o',
             'n']
           (Xml.xml1element ['t', 'y', 'p', 'e']
             (Xml.xml2elements
               ['p', 'o', 'l', 'y', 'n', 'o', 'm', 'i', 'a', 'l']
               (Xml.xml1element ['d', 'o', 'm', 'a', 'i', 'n']
                 (Xml.xmlOptions
                   [(['n', 'a', 't', 'u', 'r', 'a', 'l', 's'],
                      Xml.xml0elements ['n', 'a', 't', 'u', 'r', 'a', 'l', 's']
                        Xml.Natural),
                     (['a', 'r', 'c', 't', 'i', 'c'],
                       Xml.xml0elements ['a', 'r', 'c', 't', 'i', 'c']
                         Xml.Arctic),
                     (['a', 'r', 'c', 't', 'i', 'c', 'B', 'e', 'l', 'o', 'w',
                        'Z', 'e', 'r', 'o'],
                       Xml.xml0elements
                         ['a', 'r', 'c', 't', 'i', 'c', 'B', 'e', 'l', 'o', 'w',
                           'Z', 'e', 'r', 'o']
                         Xml.Arctic),
                     (['m', 'a', 't', 'r', 'i', 'c', 'e', 's'],
                       Xml.xml3elements ['m', 'a', 't', 'r', 'i', 'c', 'e', 's']
                         (Xml.xml2nat
                           ['d', 'i', 'm', 'e', 'n', 's', 'i', 'o', 'n'])
                         (Xml.xml2nat
                           ['s', 't', 'r', 'i', 'c', 't', 'D', 'i', 'm', 'e',
                             'n', 's', 'i', 'o', 'n'])
                         (Xml.xml1element ['d', 'o', 'm', 'a', 'i', 'n']
                           (Xml.xmlOptions
                             [(['n', 'a', 't', 'u', 'r', 'a', 'l', 's'],
                                Xml.xml0elements
                                  ['n', 'a', 't', 'u', 'r', 'a', 'l', 's']
                                  True),
                               (['a', 'r', 'c', 't', 'i', 'c'],
                                 Xml.xml0elements ['a', 'r', 'c', 't', 'i', 'c']
                                   False),
                               (['a', 'r', 'c', 't', 'i', 'c', 'B', 'e', 'l',
                                  'o', 'w', 'Z', 'e', 'r', 'o'],
                                 Xml.xml0elements
                                   ['a', 'r', 'c', 't', 'i', 'c', 'B', 'e', 'l',
                                     'o', 'w', 'Z', 'e', 'r', 'o']
                                   False)])
                           id)
                         (\ di sd doa ->
                           (if doa then Xml.Int_mat di else Xml.Arc_mat di)))])
                 id)
               (Xml.xml2nat ['d', 'e', 'g', 'r', 'e', 'e'])
               (\ typea uu -> typea))
             id)
           (\ a ->
             (case a of {
               Xml.Natural ->
                 Xml.xml3elements ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't']
                   xml2name (Xml.xml2nat ['a', 'r', 'i', 't', 'y'])
                   (\ x ->
                     ErrorMonad.bind (Xml.xml2poly Xml.xml2int_coeff x)
                       (Xml.poly_to_lpoly
                         (PolyOrder.get_comp_0
                           ((\ xa -> True),
                             (0, (1, ((\ aa b -> aa + b),
                                       ((\ aa b -> aa * b),
 (Orderings.maxa 0,
   ((\ xa y -> y <= xa),
     ((\ xa y -> y < xa),
       (True, (RedPairImpl.int_name, (\ xa -> True))))))))))))
                         (PolyOrder.get_comp_1
                           ((\ xa -> True),
                             (0, (1, ((\ aa b -> aa + b),
                                       ((\ aa b -> aa * b),
 (Orderings.maxa 0,
   ((\ xa y -> y <= xa),
     ((\ xa y -> y < xa),
       (True, (RedPairImpl.int_name, (\ xa -> True))))))))))))
                         (PolyOrder.get_comp_plus
                           ((\ xa -> True),
                             (0, (1, ((\ aa b -> aa + b),
                                       ((\ aa b -> aa * b),
 (Orderings.maxa 0,
   ((\ xa y -> y <= xa),
     ((\ xa y -> y < xa),
       (True, (RedPairImpl.int_name, (\ xa -> True))))))))))))
                         (PolyOrder.get_comp_times
                           ((\ xa -> True),
                             (0, (1, ((\ aa b -> aa + b),
                                       ((\ aa b -> aa * b),
 (Orderings.maxa 0,
   ((\ xa y -> y <= xa),
     ((\ xa y -> y < xa),
       (True, (RedPairImpl.int_name, (\ xa -> True))))))))))))))
                   (\ f aa poly ->
                     Eithera.Lefta
                       (Eithera.Lefta
                         ((f, aa),
                           (fst poly,
                             Xml.fitLength
                               (PolyOrder.get_comp_0
                                 ((\ x -> True),
                                   (0, (1,
 ((\ ab b -> ab + b),
   ((\ ab b -> ab * b),
     (Orderings.maxa 0,
       ((\ x y -> y <= x),
         ((\ x y -> y < x),
           (True, (RedPairImpl.int_name, (\ x -> True))))))))))))
                               aa (snd poly)))));
               Xml.Arctic ->
                 Xml.xml3elements ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't']
                   xml2name (Xml.xml2nat ['a', 'r', 'i', 't', 'y'])
                   (\ x ->
                     ErrorMonad.bind (Xml.xml2poly Xml.xml2arctic_coeff x)
                       (Xml.poly_to_lpoly
                         (PolyOrder.get_comp_0
                           ((\ xa -> True),
                             (RedPairImpl.MinInfty,
                               (RedPairImpl.Num_arc 0,
                                 (RedPairImpl.plus_arctic,
                                   (RedPairImpl.mult_arctic,
                                     (id, (RedPairImpl.geq_arctic,
    (RedPairImpl.gt_arctic,
      (False, (RedPairImpl.arctic_name, RedPairImpl.pos_arctic)))))))))))
                         (PolyOrder.get_comp_1
                           ((\ xa -> True),
                             (RedPairImpl.MinInfty,
                               (RedPairImpl.Num_arc 0,
                                 (RedPairImpl.plus_arctic,
                                   (RedPairImpl.mult_arctic,
                                     (id, (RedPairImpl.geq_arctic,
    (RedPairImpl.gt_arctic,
      (False, (RedPairImpl.arctic_name, RedPairImpl.pos_arctic)))))))))))
                         (PolyOrder.get_comp_plus
                           ((\ xa -> True),
                             (RedPairImpl.MinInfty,
                               (RedPairImpl.Num_arc 0,
                                 (RedPairImpl.plus_arctic,
                                   (RedPairImpl.mult_arctic,
                                     (id, (RedPairImpl.geq_arctic,
    (RedPairImpl.gt_arctic,
      (False, (RedPairImpl.arctic_name, RedPairImpl.pos_arctic)))))))))))
                         (PolyOrder.get_comp_times
                           ((\ xa -> True),
                             (RedPairImpl.MinInfty,
                               (RedPairImpl.Num_arc 0,
                                 (RedPairImpl.plus_arctic,
                                   (RedPairImpl.mult_arctic,
                                     (id, (RedPairImpl.geq_arctic,
    (RedPairImpl.gt_arctic,
      (False, (RedPairImpl.arctic_name, RedPairImpl.pos_arctic)))))))))))))
                   (\ f aa poly ->
                     Eithera.Lefta
                       (Eithera.Righta
                         ((f, aa),
                           (fst poly,
                             Xml.fitLength
                               (PolyOrder.get_comp_0
                                 ((\ x -> True),
                                   (RedPairImpl.MinInfty,
                                     (RedPairImpl.Num_arc 0,
                                       (RedPairImpl.plus_arctic,
 (RedPairImpl.mult_arctic,
   (id, (RedPairImpl.geq_arctic,
          (RedPairImpl.gt_arctic,
            (False, (RedPairImpl.arctic_name, RedPairImpl.pos_arctic)))))))))))
                               aa (snd poly)))));
               Xml.Int_mat n ->
                 Xml.xml3elements ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't']
                   xml2name (Xml.xml2nat ['a', 'r', 'i', 't', 'y'])
                   (\ x ->
                     ErrorMonad.bind
                       (Xml.xml2poly (Xml.xml2mat_coeff Xml.xml2int_coeff) x)
                       (Xml.poly_to_lpoly
                         (PolyOrder.get_comp_0
                           (Matrix.wf_mat_rcI
                              (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                              (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                             (Matrix.mat0I 0
                                (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                                (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                               (Matrix.mat1I 0 1
                                  (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                 (Matrix.mat_plusI (\ aa b -> aa + b),
                                   (Matrix.mat_mulI 0 (\ aa b -> aa + b)
                                      (\ aa b -> aa * b)
                                      (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                     (Matrix.mat_max0I (Orderings.maxa 0),
                                       (Matrix.mat_geI (\ xa y -> y <= xa),
 (Matrix.mat_gtI (\ xa y -> y <= xa) (\ xa y -> y < xa),
   (True,
     (RedPairImpl.int_mat_name, Matrix.mat_arc_posI (\ xa -> True))))))))))))
                         (PolyOrder.get_comp_1
                           (Matrix.wf_mat_rcI
                              (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                              (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                             (Matrix.mat0I 0
                                (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                                (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                               (Matrix.mat1I 0 1
                                  (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                 (Matrix.mat_plusI (\ aa b -> aa + b),
                                   (Matrix.mat_mulI 0 (\ aa b -> aa + b)
                                      (\ aa b -> aa * b)
                                      (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                     (Matrix.mat_max0I (Orderings.maxa 0),
                                       (Matrix.mat_geI (\ xa y -> y <= xa),
 (Matrix.mat_gtI (\ xa y -> y <= xa) (\ xa y -> y < xa),
   (True,
     (RedPairImpl.int_mat_name, Matrix.mat_arc_posI (\ xa -> True))))))))))))
                         (PolyOrder.get_comp_plus
                           (Matrix.wf_mat_rcI
                              (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                              (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                             (Matrix.mat0I 0
                                (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                                (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                               (Matrix.mat1I 0 1
                                  (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                 (Matrix.mat_plusI (\ aa b -> aa + b),
                                   (Matrix.mat_mulI 0 (\ aa b -> aa + b)
                                      (\ aa b -> aa * b)
                                      (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                     (Matrix.mat_max0I (Orderings.maxa 0),
                                       (Matrix.mat_geI (\ xa y -> y <= xa),
 (Matrix.mat_gtI (\ xa y -> y <= xa) (\ xa y -> y < xa),
   (True,
     (RedPairImpl.int_mat_name, Matrix.mat_arc_posI (\ xa -> True))))))))))))
                         (PolyOrder.get_comp_times
                           (Matrix.wf_mat_rcI
                              (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                              (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                             (Matrix.mat0I 0
                                (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                                (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                               (Matrix.mat1I 0 1
                                  (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                 (Matrix.mat_plusI (\ aa b -> aa + b),
                                   (Matrix.mat_mulI 0 (\ aa b -> aa + b)
                                      (\ aa b -> aa * b)
                                      (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                     (Matrix.mat_max0I (Orderings.maxa 0),
                                       (Matrix.mat_geI (\ xa y -> y <= xa),
 (Matrix.mat_gtI (\ xa y -> y <= xa) (\ xa y -> y < xa),
   (True,
     (RedPairImpl.int_mat_name, Matrix.mat_arc_posI (\ xa -> True))))))))))))))
                   (\ f aa poly ->
                     Eithera.Righta
                       (Eithera.Lefta
                         ((f, aa),
                           (fst poly,
                             Xml.fitLength
                               (PolyOrder.get_comp_0
                                 (Matrix.wf_mat_rcI
                                    (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                                    (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                   (Matrix.mat0I 0
                                      (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                                      (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                     (Matrix.mat1I 0 1
(Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                       (Matrix.mat_plusI (\ ab b -> ab + b),
 (Matrix.mat_mulI 0 (\ ab b -> ab + b) (\ ab b -> ab * b)
    (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
   (Matrix.mat_max0I (Orderings.maxa 0),
     (Matrix.mat_geI (\ x y -> y <= x),
       (Matrix.mat_gtI (\ x y -> y <= x) (\ x y -> y < x),
         (True,
           (RedPairImpl.int_mat_name,
             Matrix.mat_arc_posI (\ x -> True))))))))))))
                               aa (snd poly)))));
               Xml.Arc_mat n ->
                 Xml.xml3elements ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't']
                   xml2name (Xml.xml2nat ['a', 'r', 'i', 't', 'y'])
                   (\ x ->
                     ErrorMonad.bind
                       (Xml.xml2poly (Xml.xml2mat_coeff Xml.xml2arctic_coeff) x)
                       (Xml.poly_to_lpoly
                         (PolyOrder.get_comp_0
                           (Matrix.wf_mat_rcI
                              (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                              (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                             (Matrix.mat0I RedPairImpl.MinInfty
                                (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                                (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                               (Matrix.mat1I RedPairImpl.MinInfty
                                  (RedPairImpl.Num_arc 0)
                                  (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                 (Matrix.mat_plusI RedPairImpl.plus_arctic,
                                   (Matrix.mat_mulI RedPairImpl.MinInfty
                                      RedPairImpl.plus_arctic
                                      RedPairImpl.mult_arctic
                                      (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                     (Matrix.mat_max0I id,
                                       (Matrix.mat_geI RedPairImpl.geq_arctic,
 (Matrix.mat_geI RedPairImpl.gt_arctic,
   (False,
     (RedPairImpl.arc_mat_name,
       Matrix.mat_arc_posI RedPairImpl.pos_arctic)))))))))))
                         (PolyOrder.get_comp_1
                           (Matrix.wf_mat_rcI
                              (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                              (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                             (Matrix.mat0I RedPairImpl.MinInfty
                                (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                                (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                               (Matrix.mat1I RedPairImpl.MinInfty
                                  (RedPairImpl.Num_arc 0)
                                  (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                 (Matrix.mat_plusI RedPairImpl.plus_arctic,
                                   (Matrix.mat_mulI RedPairImpl.MinInfty
                                      RedPairImpl.plus_arctic
                                      RedPairImpl.mult_arctic
                                      (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                     (Matrix.mat_max0I id,
                                       (Matrix.mat_geI RedPairImpl.geq_arctic,
 (Matrix.mat_geI RedPairImpl.gt_arctic,
   (False,
     (RedPairImpl.arc_mat_name,
       Matrix.mat_arc_posI RedPairImpl.pos_arctic)))))))))))
                         (PolyOrder.get_comp_plus
                           (Matrix.wf_mat_rcI
                              (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                              (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                             (Matrix.mat0I RedPairImpl.MinInfty
                                (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                                (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                               (Matrix.mat1I RedPairImpl.MinInfty
                                  (RedPairImpl.Num_arc 0)
                                  (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                 (Matrix.mat_plusI RedPairImpl.plus_arctic,
                                   (Matrix.mat_mulI RedPairImpl.MinInfty
                                      RedPairImpl.plus_arctic
                                      RedPairImpl.mult_arctic
                                      (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                     (Matrix.mat_max0I id,
                                       (Matrix.mat_geI RedPairImpl.geq_arctic,
 (Matrix.mat_geI RedPairImpl.gt_arctic,
   (False,
     (RedPairImpl.arc_mat_name,
       Matrix.mat_arc_posI RedPairImpl.pos_arctic)))))))))))
                         (PolyOrder.get_comp_times
                           (Matrix.wf_mat_rcI
                              (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                              (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                             (Matrix.mat0I RedPairImpl.MinInfty
                                (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                                (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                               (Matrix.mat1I RedPairImpl.MinInfty
                                  (RedPairImpl.Num_arc 0)
                                  (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                 (Matrix.mat_plusI RedPairImpl.plus_arctic,
                                   (Matrix.mat_mulI RedPairImpl.MinInfty
                                      RedPairImpl.plus_arctic
                                      RedPairImpl.mult_arctic
                                      (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                     (Matrix.mat_max0I id,
                                       (Matrix.mat_geI RedPairImpl.geq_arctic,
 (Matrix.mat_geI RedPairImpl.gt_arctic,
   (False,
     (RedPairImpl.arc_mat_name,
       Matrix.mat_arc_posI RedPairImpl.pos_arctic)))))))))))))
                   (\ f aa poly ->
                     Eithera.Righta
                       (Eithera.Righta
                         ((f, aa),
                           (fst poly,
                             Xml.fitLength
                               (PolyOrder.get_comp_0
                                 (Matrix.wf_mat_rcI
                                    (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                                    (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                   (Matrix.mat0I RedPairImpl.MinInfty
                                      (Nat.Suc (Nat.minus_nat n Nat.one_nat))
                                      (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                     (Matrix.mat1I RedPairImpl.MinInfty
(RedPairImpl.Num_arc 0) (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
                                       (Matrix.mat_plusI
  RedPairImpl.plus_arctic,
 (Matrix.mat_mulI RedPairImpl.MinInfty RedPairImpl.plus_arctic
    RedPairImpl.mult_arctic (Nat.Suc (Nat.minus_nat n Nat.one_nat)),
   (Matrix.mat_max0I id,
     (Matrix.mat_geI RedPairImpl.geq_arctic,
       (Matrix.mat_geI RedPairImpl.gt_arctic,
         (False,
           (RedPairImpl.arc_mat_name,
             Matrix.mat_arc_posI RedPairImpl.pos_arctic)))))))))))
                               aa (snd poly)))));
             }))
           (\ typea pia ->
             (case typea of {
               Xml.Natural ->
                 RedPairImpl.Int_carrier
                   (List.mapa (Eithera.lefta . Eithera.lefta) pia);
               Xml.Arctic ->
                 RedPairImpl.Arctic_carrier
                   (List.mapa (Eithera.righta . Eithera.lefta) pia);
               Xml.Int_mat n ->
                 RedPairImpl.Int_mat_carrier (Nat.minus_nat n Nat.one_nat)
                   (List.mapa (Eithera.lefta . Eithera.righta) pia);
               Xml.Arc_mat n ->
                 RedPairImpl.Arc_mat_carrier (Nat.minus_nat n Nat.one_nat)
                   (List.mapa (Eithera.righta . Eithera.righta) pia);
             })))
         id)];

xml2ndChoice ::
  forall a b c.
    [Char] ->
      (Xml.Xml -> Eithera.Eithera [Char] a) ->
        [Char] ->
          (Xml.Xml -> Eithera.Eithera [Char] b) ->
            (a -> Maybe b -> c) -> Xml.Xml -> Eithera.Eithera [Char] c;
xml2ndChoice reqName p1 choiceName p2 combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts &&
          (Nat.less_eq_nat (Integer.nat 2) (List.size_list cs) &&
            Option.is_none text))
    then ErrorMonad.bind (p1 (List.nth cs Nat.Zero_nat))
           (\ a ->
             ErrorMonad.bind (Xml.xml2bool choiceName (List.nth cs Nat.one_nat))
               (\ choice ->
                 (if choice
                   then ErrorMonad.bind
                          (p2 (List.nth cs
                                (Nat.minus_nat (List.size_list cs)
                                  Nat.one_nat)))
                          (\ b -> ErrorMonad.returna (combi a (Just b)))
                   else ErrorMonad.returna (combi a Nothing))))
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml2dpProof ::
  forall a.
    Nat.Nat ->
      (Nat.Nat ->
        Xml.Xml -> Eithera.Eithera [Char] (SemanticLabeling.Lab a [Nat.Nat]))
        -> Xml.Xml ->
             Eithera.Eithera [Char] (ProofChecker.DPProof a [Nat.Nat] [Char]);
xml2dpProof ld xml2name x =
  Xml.xmlOptions
    [(['p', 'I', 's', 'E', 'm', 'p', 't', 'y'],
       Xml.xml0elements ['p', 'I', 's', 'E', 'm', 'p', 't', 'y']
         ProofChecker.PisEmpty),
      (['d', 'e', 'p', 'G', 'r', 'a', 'p', 'h', 'P', 'r', 'o', 'c'],
        Xml.xmlManyElements
          ['d', 'e', 'p', 'G', 'r', 'a', 'p', 'h', 'P', 'r', 'o', 'c']
          (Xml.xml2ndChoice ['c', 'o', 'm', 'p', 'o', 'n', 'e', 'n', 't']
            (Xml.xml1element ['d', 'p', 's'] (Xml.xml2rules (xml2name ld)) id)
            ['r', 'e', 'a', 'l', 'S', 'c', 'c'] (Xml.xml2dpProof ld xml2name)
            (\ dps prfOpt -> (prfOpt, dps)))
          ProofChecker.DepGraphProc),
      (['r', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c'],
        Xml.xml3elements ['r', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c']
          (Xml.xml2redpair (xml2name ld))
          (Xml.xml1element ['d', 'p', 's'] (Xml.xml2rules (xml2name ld)) id)
          (Xml.xml2dpProof ld xml2name) ProofChecker.RedPairProc),
      (['r', 'e', 'd', 'P', 'a', 'i', 'r', 'U', 'r', 'P', 'r', 'o', 'c'],
        Xml.xml4elements
          ['r', 'e', 'd', 'P', 'a', 'i', 'r', 'U', 'r', 'P', 'r', 'o', 'c']
          (Xml.xml2redpair (xml2name ld))
          (Xml.xml1element ['d', 'p', 's'] (Xml.xml2rules (xml2name ld)) id)
          (Xml.xml1element
            ['u', 's', 'a', 'b', 'l', 'e', 'R', 'u', 'l', 'e', 's']
            (Xml.xml2rules (xml2name ld)) id)
          (Xml.xml2dpProof ld xml2name) ProofChecker.RedPairUrProc),
      (['m', 'o', 'n', 'o', 'R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o',
         'c'],
        Xml.xml4elements
          ['m', 'o', 'n', 'o', 'R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o',
            'c']
          (Xml.xml2redpair (xml2name ld))
          (Xml.xml1element ['d', 'p', 's'] (Xml.xml2rules (xml2name ld)) id)
          (Xml.xml1element ['t', 'r', 's'] (Xml.xml2rules (xml2name ld)) id)
          (Xml.xml2dpProof ld xml2name) ProofChecker.MonoRedPairProc),
      (['m', 'o', 'n', 'o', 'R', 'e', 'd', 'P', 'a', 'i', 'r', 'U', 'r', 'P',
         'r', 'o', 'c'],
        Xml.xml5elements
          ['m', 'o', 'n', 'o', 'R', 'e', 'd', 'P', 'a', 'i', 'r', 'U', 'r', 'P',
            'r', 'o', 'c']
          (Xml.xml2redpair (xml2name ld))
          (Xml.xml1element ['d', 'p', 's'] (Xml.xml2rules (xml2name ld)) id)
          (Xml.xml1element ['t', 'r', 's'] (Xml.xml2rules (xml2name ld)) id)
          (Xml.xml1element
            ['u', 's', 'a', 'b', 'l', 'e', 'R', 'u', 'l', 'e', 's']
            (Xml.xml2rules (xml2name ld)) id)
          (Xml.xml2dpProof ld xml2name) ProofChecker.MonoRedPairUrProc),
      (['s', 'e', 'm', 'l', 'a', 'b', 'P', 'r', 'o', 'c'],
        Xml.xml4elements ['s', 'e', 'm', 'l', 'a', 'b', 'P', 'r', 'o', 'c']
          (Xml.xml2sl_inter (xml2name ld))
          (Xml.xml1element ['d', 'p', 's']
            (Xml.xml2rules (xml2name (Nat.Suc ld))) id)
          (Xml.xml1element ['t', 'r', 's']
            (Xml.xml2rules (xml2name (Nat.Suc ld))) id)
          (Xml.xml2dpProof (Nat.Suc ld) xml2name)
          (\ a ->
            (case a of {
              Nothing -> ProofChecker.RootlabProc;
              Just i -> ProofChecker.SemlabProc i Xml.id_labeling;
            }))),
      (['u', 'n', 'l', 'a', 'b', 'P', 'r', 'o', 'c'],
        Xml.xml3elements ['u', 'n', 'l', 'a', 'b', 'P', 'r', 'o', 'c']
          (Xml.xml1element ['d', 'p', 's']
            (Xml.xml2rules (xml2name (Nat.minus_nat ld Nat.one_nat))) id)
          (Xml.xml1element ['t', 'r', 's']
            (Xml.xml2rules (xml2name (Nat.minus_nat ld Nat.one_nat))) id)
          (Xml.xml2dpProof (Nat.minus_nat ld Nat.one_nat) xml2name)
          ProofChecker.UnlabProc),
      (['s', 'u', 'b', 't', 'e', 'r', 'm', 'P', 'r', 'o', 'c'],
        Xml.xml3elements ['s', 'u', 'b', 't', 'e', 'r', 'm', 'P', 'r', 'o', 'c']
          (Xml.xml2sp (xml2name ld))
          (Xml.xml1element ['d', 'p', 's'] (Xml.xml2rules (xml2name ld)) id)
          (Xml.xml2dpProof ld xml2name) ProofChecker.SpscProc),
      (['s', 'i', 'z', 'e', 'C', 'h', 'a', 'n', 'g', 'e', 'C', 'r', 'i', 't',
         'e', 'r', 'i', 'o', 'n'],
        Xml.xml1manyElements
          ['s', 'i', 'z', 'e', 'C', 'h', 'a', 'n', 'g', 'e', 'C', 'r', 'i', 't',
            'e', 'r', 'i', 'o', 'n']
          (Xml.xmlOptions
            [(['s', 'u', 'b', 't', 'e', 'r', 'm', 'C', 'r', 'i', 't', 'e', 'r',
                'i', 'o', 'n'],
               Xml.xml0elements
                 ['s', 'u', 'b', 't', 'e', 'r', 'm', 'C', 'r', 'i', 't', 'e',
                   'r', 'i', 'o', 'n']
                 Nothing),
              (['r', 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n', 'P', 'a', 'i',
                 'r'],
                Xml.xml2elements
                  ['r', 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n', 'P', 'a', 'i',
                    'r']
                  (Xml.xml2redpair (xml2name ld))
                  (Xml.xml1element
                    ['u', 's', 'a', 'b', 'l', 'e', 'R', 'u', 'l', 'e', 's']
                    (Xml.xml2rules (xml2name ld)) id)
                  (\ redp ur -> Just (redp, ur)))])
          (Xml.xml2scg (xml2name ld))
          (\ a ->
            (case a of {
              Nothing -> ProofChecker.SizeChangeSubTerm;
              Just redp_ur ->
                ProofChecker.SizeChangeRedPair (fst redp_ur) (snd redp_ur);
            })))]
    x;

xml2trsInput ::
  forall f.
    (Xml.Xml -> Eithera.Eithera [Char] f) ->
      Xml.Xml ->
        Eithera.Eithera [Char] [(Term.Term f [Char], Term.Term f [Char])];
xml2trsInput xml2name =
  Xml.xml1element ['i', 'n', 'p', 'u', 't']
    (Xml.xml1element ['t', 'r', 's', 'I', 'n', 'p', 'u', 't']
      (Xml.xml1element ['t', 'r', 's'] (Xml.xml2rules xml2name) id) id)
    id;

xml2trsProof ::
  forall a.
    (Xml.Xml -> Eithera.Eithera [Char] (SemanticLabeling.Lab a [Nat.Nat])) ->
      (Xml.Xml ->
        Eithera.Eithera [Char] (Trs.Shp (SemanticLabeling.Lab a [Nat.Nat])))
        -> (Nat.Nat ->
             Xml.Xml ->
               Eithera.Eithera [Char]
                 (SemanticLabeling.Lab
                   (Trs.Shp (SemanticLabeling.Lab a [Nat.Nat])) [Nat.Nat]))
             -> Xml.Xml ->
                  Eithera.Eithera [Char]
                    (ProofChecker.TRSProof a [Nat.Nat] [Char]);
xml2trsProof xml2name xml2sname xml2lsname x =
  Xml.xmlOptions
    [(['r', 'I', 's', 'E', 'm', 'p', 't', 'y'],
       Xml.xml0elements ['r', 'I', 's', 'E', 'm', 'p', 't', 'y']
         ProofChecker.RisEmpty),
      (['d', 'p', 'T', 'r', 'a', 'n', 's'],
        Xml.xml3elements ['d', 'p', 'T', 'r', 'a', 'n', 's']
          (Xml.xml1element ['d', 'p', 's'] (Xml.xml2rules xml2sname) id)
          (Xml.xml2bool
            ['m', 'a', 'r', 'k', 'e', 'd', 'S', 'y', 'm', 'b', 'o', 'l', 's'])
          (Xml.xml2dpProof Nat.Zero_nat xml2lsname)
          (\ dps uu -> ProofChecker.DPTrans dps)),
      (['r', 'u', 'l', 'e', 'R', 'e', 'm', 'o', 'v', 'a', 'l'],
        Xml.xml3elements ['r', 'u', 'l', 'e', 'R', 'e', 'm', 'o', 'v', 'a', 'l']
          (Xml.xml2redpair xml2name)
          (Xml.xml1element ['t', 'r', 's'] (Xml.xml2rules xml2name) id)
          (Xml.xml2trsProof xml2name xml2sname xml2lsname)
          ProofChecker.RuleRemoval),
      (['u', 'n', 'l', 'a', 'b'],
        Xml.xml2elements ['u', 'n', 'l', 'a', 'b']
          (Xml.xml1element ['t', 'r', 's'] (Xml.xml2rules xml2name) id)
          (Xml.xml2trsProof xml2name xml2sname xml2lsname) ProofChecker.Unlab),
      (['s', 'e', 'm', 'l', 'a', 'b'],
        Xml.xml3elements ['s', 'e', 'm', 'l', 'a', 'b']
          (Xml.xml2sl_inter xml2name)
          (Xml.xml1element ['t', 'r', 's'] (Xml.xml2rules xml2name) id)
          (Xml.xml2trsProof xml2name xml2sname xml2lsname)
          (\ a ->
            (case a of {
              Nothing -> ProofChecker.Rootlab;
              Just i -> ProofChecker.Semlab i Xml.id_labeling;
            })))]
    x;

xml2plainName ::
  Xml.Xml -> Eithera.Eithera [Char] (SemanticLabeling.Lab [Char] [Nat.Nat]);
xml2plainName x =
  Xml.xmlOptions
    [(['n', 'a', 'm', 'e'],
       Xml.xmlChange (Xml.xml2text ['n', 'a', 'm', 'e'])
         SemanticLabeling.UnLab),
      (['l', 'a', 'b', 'e', 'l', 'e', 'd', 'S', 'y', 'm', 'b', 'o', 'l'],
        Xml.xml2elements
          ['l', 'a', 'b', 'e', 'l', 'e', 'd', 'S', 'y', 'm', 'b', 'o', 'l']
          Xml.xml2plainName (Xml.xml2label Xml.xml2plainName)
          (\ f a ->
            (case a of {
              Eithera.Lefta aa -> SemanticLabeling.Lab f aa;
              Eithera.Righta aa -> SemanticLabeling.FunLab f aa;
            })))]
    x;

xml2sharpName ::
  Xml.Xml ->
    Eithera.Eithera [Char] (Trs.Shp (SemanticLabeling.Lab [Char] [Nat.Nat]));
xml2sharpName x =
  (if Xml.xmlGetName x == ['s', 'h', 'a', 'r', 'p']
    then Xml.xml1element ['s', 'h', 'a', 'r', 'p'] Xml.xml2plainName Trs.Sharp x
    else Xml.xmlChange Xml.xml2plainName Trs.Plain x);

terminationGoal :: Xml.Xml;
terminationGoal =
  Xml.Xml ['g', 'o', 'a', 'l'] []
    [Xml.Xml ['t', 'e', 'r', 'm', 'i', 'n', 'a', 't', 'i', 'o', 'n'] [] []
       Nothing]
    Nothing;

xml2substitution ::
  forall a.
    (Xml.Xml -> Eithera.Eithera [Char] a) ->
      Xml.Xml ->
        Eithera.Eithera [Char] [(Term.Term a [Char], Term.Term a [Char])];
xml2substitution xml2name =
  Xml.xmlManyElements
    ['s', 'u', 'b', 's', 't', 'i', 't', 'u', 't', 'i', 'o', 'n']
    (Xml.xml2elements ['s', 'u', 'b', 's', 't', 'E', 'n', 't', 'r', 'y']
      Xml.xml2var (Xml.xml2term xml2name) (\ a b -> (a, b)))
    id;

xmlMany2Elements ::
  forall a b c d.
    [Char] ->
      (Xml.Xml -> Eithera.Eithera [Char] a) ->
        (Xml.Xml -> Eithera.Eithera [Char] b) ->
          (Xml.Xml -> Eithera.Eithera [Char] c) ->
            ([a] -> b -> c -> d) -> Xml.Xml -> Eithera.Eithera [Char] d;
xmlMany2Elements reqName p1 p2 p3 combi (Xml.Xml name atts cs text) =
  let {
    ds = List.rev cs;
  } in (if name == reqName &&
             (List.nulla atts &&
               (Option.is_none text &&
                 Nat.less_eq_nat (Integer.nat 2) (List.size_list cs)))
         then ErrorMonad.bind
                (Xml.mapMonad p1 (List.rev (List.tl (List.tl ds))))
                (\ a ->
                  ErrorMonad.bind (p2 (List.nth ds Nat.one_nat))
                    (\ b ->
                      ErrorMonad.bind (p3 (List.nth ds Nat.Zero_nat))
                        (ErrorMonad.returna . combi a b)))
         else ErrorMonad.errora
                (List.concata
                  [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r',
                     'a', 'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ',
                     'f', 'o', 'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm',
                     'l', ' ', 'e', 'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e',
                     'x', 'p', 'e', 'c', 't', 'e', 'd', ' '],
                    reqName, [')'], Show.newline,
                    Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml2trsDisproof ::
  forall f.
    (Xml.Xml -> Eithera.Eithera [Char] f) ->
      Xml.Xml -> Eithera.Eithera [Char] (ProofChecker.TRSDisproof f [Char]);
xml2trsDisproof xml2name =
  Xml.xmlOptions
    [(['v', 'a', 'r', 'i', 'a', 'b', 'l', 'e', 'C', 'o', 'n', 'd', 'i', 't',
        'i', 'o', 'n'],
       Xml.xml0elements
         ['v', 'a', 'r', 'i', 'a', 'b', 'l', 'e', 'C', 'o', 'n', 'd', 'i', 't',
           'i', 'o', 'n']
         ProofChecker.NotWellFormed),
      (['l', 'o', 'o', 'p'],
        Xml.xmlMany2Elements ['l', 'o', 'o', 'p']
          (Xml.xml3elements ['r', 'e', 'd', 'e', 'x'] (Xml.xml2term xml2name)
            Xml.xml2pos (Xml.xml2rule xml2name) (\ t p r -> t))
          (Xml.xml2substitution xml2name) (Xml.xml2context xml2name)
          (\ ts sigma c -> ProofChecker.Loop (ts, (c, sigma))))];

nonTerminationGoal :: Xml.Xml;
nonTerminationGoal =
  Xml.Xml ['g', 'o', 'a', 'l'] []
    [Xml.Xml
       ['n', 'o', 'n', 'T', 'e', 'r', 'm', 'i', 'n', 'a', 't', 'i', 'o', 'n'] []
       [] Nothing]
    Nothing;

xml2certProblem ::
  forall a.
    (Xml.Xml -> Eithera.Eithera [Char] a) ->
      (Xml.Xml -> Eithera.Eithera [Char] (SemanticLabeling.Lab a [Nat.Nat])) ->
        (Xml.Xml ->
          Eithera.Eithera [Char] (Trs.Shp (SemanticLabeling.Lab a [Nat.Nat])))
          -> (Nat.Nat ->
               Xml.Xml ->
                 Eithera.Eithera [Char]
                   (SemanticLabeling.Lab
                     (Trs.Shp (SemanticLabeling.Lab a [Nat.Nat])) [Nat.Nat]))
               -> Xml.Xml ->
                    Eithera.Eithera [Char]
                      (ProofChecker.CertificationProblem a [Nat.Nat] [Char]);
xml2certProblem xml2uname xml2name xml2sname xml2lsname
  (Xml.Xml name uu cs text) =
  (if name ==
        ['c', 'e', 'r', 't', 'i', 'f', 'i', 'c', 'a', 't', 'i', 'o', 'n', 'P',
          'r', 'o', 'b', 'l', 'e', 'm']
        && (Option.is_none text &&
             Nat.less_eq_nat (Integer.nat 3) (List.size_list cs))
    then ErrorMonad.bind (Xml.xml2trsInput xml2uname (List.nth cs Nat.one_nat))
           (\ trs ->
             (if List.nth cs Nat.Zero_nat == Xml.terminationGoal
               then ErrorMonad.bind
                      (Xml.xml1element ['p', 'r', 'o', 'o', 'f']
                        (Xml.xml2trsProof xml2name xml2sname xml2lsname) id
                        (List.nth cs (Integer.nat 2)))
                      (\ p ->
                        ErrorMonad.returna
                          (ProofChecker.CertificationProblem True trs
                            (Eithera.Lefta p)))
               else (if List.nth cs Nat.Zero_nat == Xml.nonTerminationGoal
                      then ErrorMonad.bind
                             (Xml.xml1element ['p', 'r', 'o', 'o', 'f']
                               (Xml.xml2trsDisproof xml2uname) id
                               (List.nth cs (Integer.nat 2)))
                             (\ p ->
                               ErrorMonad.returna
                                 (ProofChecker.CertificationProblem False trs
                                   (Eithera.Righta p)))
                      else ErrorMonad.errora
                             ['u', 'n', 'k', 'n', 'o', 'w', 'n', ' ', 'g', 'o',
                               'a', 'l', ' ', 't', 'y', 'p', 'e', ' ', 'i', 'n',
                               ' ', 'c', 'e', 'r', 't', 'i', 'f', 'i', 'c', 'a',
                               't', 'i', 'o', 'n', ' ', 'p', 'r', 'o', 'b', 'l',
                               'e', 'm'])))
    else ErrorMonad.errora
           ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'p', 'a', 'r',
             's', 'e', ' ', 'r', 'o', 'o', 't', ' ', 'e', 'l', 'e', 'm', 'e',
             'n', 't']);

xml2labSharpName ::
  Nat.Nat ->
    Xml.Xml ->
      Eithera.Eithera [Char]
        (SemanticLabeling.Lab (Trs.Shp (SemanticLabeling.Lab [Char] [Nat.Nat]))
          [Nat.Nat]);
xml2labSharpName Nat.Zero_nat =
  Xml.xmlChange Xml.xml2sharpName SemanticLabeling.UnLab;
xml2labSharpName (Nat.Suc n) =
  Xml.xml2elements
    ['l', 'a', 'b', 'e', 'l', 'e', 'd', 'S', 'y', 'm', 'b', 'o', 'l']
    (Xml.xml2labSharpName n) (Xml.xml2label (Xml.xml2labSharpName n))
    (\ f a ->
      (case a of {
        Eithera.Lefta aa -> SemanticLabeling.Lab f aa;
        Eithera.Righta aa -> SemanticLabeling.FunLab f aa;
      }));

xmldoc2certProblem ::
  Xml.Xmldoc ->
    Eithera.Eithera [Char]
      (ProofChecker.CertificationProblem [Char] [Nat.Nat] [Char]);
xmldoc2certProblem (Xml.Xmldoc header xml) =
  ProofChecker.debug ['0']
    ['p', 'a', 'r', 's', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 't', 'o', ' ',
      'c', 'e', 'r', 't', 'i', 'f', 'i', 'c', 'a', 't', 'i', 'o', 'n', ' ', 'p',
      'r', 'o', 'b', 'l', 'e', 'm']
    (Xml.xml2certProblem (Xml.xml2text ['n', 'a', 'm', 'e']) Xml.xml2plainName
      Xml.xml2sharpName Xml.xml2labSharpName xml);

parseCertProblem ::
  [Char] ->
    Eithera.Eithera [Char]
      (ProofChecker.CertificationProblem [Char] [Nat.Nat] [Char]);
parseCertProblem s = ErrorMonad.bind (Xml.parseXMLDoc s) Xml.xmldoc2certProblem;

xml_certificationProblem ::
  forall a b c.
    (Show.Sshow a,
      Show.Sshow c) => ProofChecker.CertificationProblem a b c -> Xml.Xml;
xml_certificationProblem (ProofChecker.CertificationProblem term rls prf) =
  let {
    inp = Xml.Xml ['i', 'n', 'p', 'u', 't'] []
            [Xml.Xml ['t', 'r', 's', 'I', 'n', 'p', 'u', 't'] []
               [Xml.Xml ['t', 'r', 's'] [] [Xml.xml_rules rls] Nothing] Nothing]
            Nothing;
    goal =
      (if Eithera.isLeft prf then Xml.terminationGoal
        else Xml.nonTerminationGoal);
  } in Xml.Xml ['p', 'r', 'o', 'b', 'l', 'e', 'm'] [] [goal, inp] Nothing;

}
