<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:ceta="http://cl-informatik.uibk.ac.at/software/ceta" 
    version="1.0">
    <xsl:output method="html"/>
    <xsl:strip-space elements="ceta:arg"/>
    
    <xsl:template match="/ceta:proof">
        <xsl:variable name="mode">
            <xsl:choose>
                <xsl:when test="count(/ceta:proof/ceta:loop) + count(/ceta:proof/ceta:notWellFormed) = 0">Termination Proof</xsl:when>
                <xsl:otherwise>Nontermination Proof</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <html>
            <head>
                <title>
                    <xsl:value-of select="$mode"/>
                </title>
                <style rel="stylesheet" type="text/css">
                  .fun {
                    color: darkblue;
                  }
                  .dp_fun {
                    color: darkgreen;
                  }
                  .var {
                    color: red;
                  }
                  .label {
                    color: gray;
                  }
                </style>
            </head>
            <body>
                <h1>
                    <xsl:value-of select="$mode"/>
                </h1>
                <p>
                    <xsl:apply-templates>
                        <xsl:with-param name="indent" select="1"/>
                    </xsl:apply-templates>                        
                </p>
            </body>
        </html>
    </xsl:template>    
        
    <xsl:template match="ceta:notWellFormed">
        <xsl:param name="indent"/>
        <h3><xsl:value-of select="$indent"/> Not Well-Formed</h3>
        <p>The TRS violates one of the two variable conditions. Thus, it is not terminating.</p>
    </xsl:template>
    
    
    <xsl:template match="ceta:loop">
        <xsl:param name="indent"/>
        <xsl:variable name="context" select="count(ceta:box) = 0"/>
        <xsl:variable name="subst" select="count(ceta:substitution/ceta:substEntry) &gt; 0"/>
        <h3><xsl:value-of select="$indent"/> Loop</h3>
        The following loop proves nontermination.            
        <p>
            <table>
                <xsl:attribute name="align">center</xsl:attribute>
                <xsl:for-each select="*">
                    <xsl:if test="position() &gt; 2">
                        <tr>
                            <xsl:choose>
                                <xsl:when test="position() = 3">
                                    <td><xsl:attribute name="align">right</xsl:attribute>t</td>
                                    <td><xsl:attribute name="align">center</xsl:attribute>=</td>
                                </xsl:when>
                                <xsl:otherwise>
                                    <td/>
                                    <td><xsl:attribute name="align">center</xsl:attribute>&#8594;</td>                                    
                                </xsl:otherwise>
                            </xsl:choose>
                            <td><xsl:attribute name="align">left</xsl:attribute><xsl:apply-templates select="."/></td>
                        </tr>
                    </xsl:if>
                </xsl:for-each>
                <tr>
                    <td/>
                    <td><xsl:attribute name="align">center</xsl:attribute>&#8594;</td>                                    
                    <td><xsl:attribute name="align">left</xsl:attribute>
                        <xsl:if test="$context">C[</xsl:if>t<xsl:if test="$subst">&#963;</xsl:if><xsl:if test="$context">]</xsl:if>
                    </td>
                </tr>
            </table>          
            <xsl:if test="$subst or $context">
                where
            </xsl:if>
            <xsl:if test="$subst">
                &#963; = <xsl:apply-templates select="ceta:substitution"/>
            </xsl:if>
            <xsl:if test="$subst and $context">
                and
            </xsl:if>            
            <xsl:if test="$context">
                C = <xsl:apply-templates select="*[2]"/>
            </xsl:if>
        </p>
    </xsl:template>
    
    <xsl:template match="ceta:type">
        <xsl:choose>
            <xsl:when test="count(ceta:linearPolynomial) &gt; 0">
                linear polynomial
            </xsl:when>
            <xsl:otherwise>
                unknown
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="ceta:domain">
        <xsl:choose>
            <xsl:when test="count(ceta:naturals) &gt; 0">
                the naturals
            </xsl:when>
            <xsl:otherwise>
                an unknown domain
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="genVars">
        <xsl:param name="n"/>
        <xsl:choose>
            <xsl:when test="$n = 0"/>
            <xsl:when test="$n = 1">
                <xsl:text>(</xsl:text><span class="var">x<sub>1</sub></span><xsl:text>)</xsl:text>
            </xsl:when>
            <xsl:when test="$n = 2">
                <xsl:text>(</xsl:text><span class="var">x<sub>1</sub></span>, <span class="var">x<sub>2</sub></span><xsl:text>)</xsl:text>
            </xsl:when>
            <xsl:when test="$n = 3">
                <xsl:text>(</xsl:text><span class="var">x<sub>1</sub></span>, <span class="var">x<sub>2</sub></span>, <span class="var">x<sub>3</sub></span><xsl:text>)</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                  <xsl:text>(</xsl:text><span class="var">x<sub>1</sub></span>,...,<span class="var">x<sub><xsl:value-of select="$n"/></sub></span><xsl:text>)</xsl:text>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="ceta:linearPolynomial">
        <xsl:if test="ceta:constant/ceta:number/text() &lt; '0'">
	  <xsl:text>max(</xsl:text>
        </xsl:if>
        <xsl:for-each select="ceta:coefficient[ceta:natural/text() != '0']">
            <xsl:if test="position() != 1"> + </xsl:if>
                <xsl:if test="ceta:natural/text() != '1'">
                    <xsl:apply-templates select="."/>      
                </xsl:if>
                <span class="var">x<sub><xsl:value-of select="position()"/></sub></span>
        </xsl:for-each>        
        <xsl:if test="ceta:constant/ceta:number/text() &lt; '0'">
            <xsl:text> </xsl:text>
            <xsl:apply-templates select="ceta:constant"/>
	    <xsl:text>,0)</xsl:text>
        </xsl:if>
        <xsl:if test="ceta:constant/ceta:number/text() &gt; '0'">
            <xsl:if test="count(ceta:coefficient[ceta:natural/text() != 0]) &gt; 0">
                +                
            </xsl:if>
            <xsl:apply-templates select="ceta:constant"/>
        </xsl:if>
        <xsl:if test="ceta:constant/ceta:number/text() = '0'">
            <xsl:if test="count(ceta:coefficient[ceta:natural/text() != 0]) = 0">
                <xsl:apply-templates select="ceta:constant"/>                
            </xsl:if>            
        </xsl:if>
    </xsl:template>
    <xsl:template match="ceta:constant">
        <xsl:apply-templates/>
    </xsl:template>
    <xsl:template match="ceta:coefficient">        
        <xsl:apply-templates/>
    </xsl:template>
    
    <xsl:template match="ceta:interpretation">
            <xsl:apply-templates select="*[1]"/> 
            interpretation over 
            <xsl:apply-templates select="*[2]"/>
            <p>
            <table>
                <xsl:attribute name="align">center</xsl:attribute>
                <xsl:for-each select="ceta:interpret">
                    <tr>
                        <td><xsl:attribute name="align">right</xsl:attribute>
                            <xsl:text>[</xsl:text><xsl:apply-templates select="ceta:name"/><xsl:call-template name="genVars">
                                <xsl:with-param name="n" select="ceta:arity"/>
                            </xsl:call-template><xsl:text>]</xsl:text>
                        </td>
                        <td><xsl:attribute name="align">center</xsl:attribute> = </td>
                        <td><xsl:attribute name="align">left</xsl:attribute><xsl:apply-templates select="ceta:function"/></td>
                    </tr>           
                </xsl:for-each>
                <tr>
                    <td><xsl:attribute name="align">right</xsl:attribute>
                        <xsl:text>[f(</xsl:text><span class="var">x<sub>1</sub></span>,...,<span class="var">x<sub>n</sub></span>)]
                    </td>
                    <td><xsl:attribute name="align">center</xsl:attribute> = </td>
                    <td><xsl:attribute name="align">left</xsl:attribute>
                        <span class="var">x<sub>1</sub></span> +  ... +  <span class="var">x<sub>n</sub></span> + 1 
                    </td>
                    <td><xsl:attribute name="align">right</xsl:attribute>
                         for all other symbols f of arity n
                    </td>                            
                </tr>
            </table>          
            </p>
    </xsl:template>
    
    <xsl:template match="ceta:dpTrans">
        <xsl:param name="indent"/>
        <h3><xsl:value-of select="$indent"/> Dependency Pair Transformation</h3>
        <xsl:choose>
        <xsl:when test="count(ceta:dps/ceta:rules/ceta:rule) &gt; 0">
          The following set of initial dependency pairs has been identified.
          <xsl:apply-templates select="ceta:dps/*"/>
          <xsl:apply-templates select="*[2]">
              <xsl:with-param name="indent" select="concat($indent, '.1')"/>
          </xsl:apply-templates>
        </xsl:when>
        <xsl:otherwise>
          The set of initial dependency pairs is empty, and hence the TRS is
          terminating. 
        </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    
    <xsl:template match="ceta:unlabProc">
        <xsl:param name="indent"/>
        <h3><xsl:value-of select="$indent"/> Unlabeling Processor</h3>
        After removing one layer of labels
        we obtain the set of pairs
        <xsl:apply-templates select="ceta:dps/*"/>
        and the set of rules        
        <xsl:apply-templates select="ceta:trs/*"/>
        <p>
            <xsl:apply-templates select="*[3]">
                <xsl:with-param name="indent" select="concat($indent, '.1')"/>
            </xsl:apply-templates>
        </p>
    </xsl:template>

    <xsl:template match="ceta:semlabProc">
        <xsl:param name="indent"/>
        <h3><xsl:value-of select="$indent"/> Semantic Labeling Processor</h3>
        The following interpretations form a model of the rules.
        <xsl:apply-templates select="ceta:model"/>
        Then we obtain the set of labeled pairs
        <xsl:apply-templates select="ceta:dps/*"/>
        and the set of labeled rules        
        <xsl:apply-templates select="ceta:trs/*"/>
        <p>
            <xsl:apply-templates select="*[4]">
                <xsl:with-param name="indent" select="concat($indent, '.1')"/>
            </xsl:apply-templates>
        </p>
    </xsl:template>
    
    <xsl:template mode="arithFun" match="ceta:natural">
        <xsl:apply-templates/>
    </xsl:template>

    <xsl:template mode="arithFun" match="ceta:variable">
        <span class="var">x<sub><xsl:apply-templates/></sub></span>
    </xsl:template>

    <xsl:template mode="arithFun" match="ceta:sum">
        <xsl:for-each select="*">
            <xsl:apply-templates select="." mode="arithFun"/>
            <xsl:if test="position() != last()"> + </xsl:if>
        </xsl:for-each>
    </xsl:template>

    <xsl:template mode="arithFun" match="ceta:prod">
        <xsl:text>(</xsl:text>
        <xsl:for-each select="*">
            <xsl:apply-templates select="." mode="arithFun"/>
            <xsl:if test="position() != last()"> * </xsl:if>
        </xsl:for-each>
        <xsl:text>)</xsl:text>        
    </xsl:template>
    
    <xsl:template mode="arithFun" match="ceta:minimum">
        <xsl:text>min(</xsl:text>
        <xsl:for-each select="*">
            <xsl:apply-templates select="." mode="arithFun"/>
            <xsl:if test="position() != last()">,</xsl:if>
        </xsl:for-each>
        <xsl:text>)</xsl:text>
    </xsl:template>

    <xsl:template mode="arithFun" match="ceta:maximum">
        <xsl:text>max(</xsl:text>
        <xsl:for-each select="*">
            <xsl:apply-templates select="." mode="arithFun"/>
            <xsl:if test="position() != last()">,</xsl:if>
        </xsl:for-each>
        <xsl:text>)</xsl:text>
    </xsl:template>
    
    <xsl:template match="ceta:model">
        <p>
          As carrier we take the set 
          <xsl:choose>
              <xsl:when test="ceta:carrierSize/text() = 1">{0}</xsl:when>
              <xsl:when test="ceta:carrierSize/text() = 2">{0,1}</xsl:when>
              <xsl:when test="ceta:carrierSize/text() = 3">{0,1,2}</xsl:when>
              <xsl:otherwise>{0,...,<xsl:value-of select="ceta:carrierSize/text() - 1"/>}</xsl:otherwise>
          </xsl:choose>
          <xsl:text>.</xsl:text>
	  Symbols are labeled by the interpretation of their arguments using the interpretations
          (modulo <xsl:value-of select="ceta:carrierSize/text()"/>):
          <p>
              <table>
                  <xsl:attribute name="align">center</xsl:attribute>
                  <xsl:for-each select="ceta:interpret">
                      <tr>
                          <td><xsl:attribute name="align">right</xsl:attribute>
                              <xsl:text>[</xsl:text><xsl:apply-templates select="ceta:name"/><xsl:call-template name="genVars">
                                  <xsl:with-param name="n" select="ceta:arity"/>
                              </xsl:call-template><xsl:text>]</xsl:text>
                          </td>
                          <td><xsl:attribute name="align">center</xsl:attribute> = </td>
                          <td><xsl:attribute name="align">left</xsl:attribute><xsl:apply-templates mode="arithFun" select="*[3]"/></td>
                      </tr>           
                  </xsl:for-each>
                  <tr>
                      <td><xsl:attribute name="align">right</xsl:attribute>
                          <xsl:text>[f(</xsl:text><span class="var">x<sub>1</sub></span>,...,<span class="var">x<sub>n</sub></span>)]
                      </td>
                      <td><xsl:attribute name="align">center</xsl:attribute> = </td>
                      <td><xsl:attribute name="align">left</xsl:attribute>
                          0 
                      </td>
                      <td><xsl:attribute name="align">right</xsl:attribute>
                          for all other symbols f of arity n
                      </td>                            
                  </tr>
              </table>          
              </p>
        </p>
    </xsl:template>
    
    <xsl:template name="ceta:ProofStep">
      <xsl:param name="indent"/>
      <xsl:param name="name"/>
      <xsl:param name="justification"/>
      <xsl:param name="pairs"/>
      <xsl:param name="urules">null</xsl:param>
      <xsl:param name="rules">null</xsl:param>
      <xsl:param name="proof"/>
      Using the <xsl:value-of select="$name"/>
      <xsl:apply-templates select="$justification"/>
      <xsl:if test="string($urules) != 'null'">
        <xsl:choose>
          <xsl:when test="count($urules) &gt; 0">
            together with the usable
            rule<xsl:if test="count($urules) &gt; 1">s</xsl:if>
            <xsl:apply-templates select="$urules/.."/>
            (w.r.t. the implicit argument filter of the reduction pair),
          </xsl:when>
          <xsl:otherwise>
            having no usable rules (w.r.t. the implicit argument filter of the
            reduction pair),
          </xsl:otherwise>
        </xsl:choose>
      </xsl:if>
      <xsl:choose>
        <xsl:when test="count($pairs) &gt; 0">
          the
          pair<xsl:if test="count($pairs) &gt; 1">s</xsl:if> 
          <xsl:apply-templates select="$pairs/.."/>
	  <xsl:if test="string($rules) != 'null'">
	    and
            <xsl:choose>
              <xsl:when test="count($rules) &gt; 0">
                the
                rule<xsl:if test="count($rules) &gt; 1">s</xsl:if>
                <xsl:apply-templates select="$rules/.."/>
              </xsl:when>
              <xsl:otherwise>
                no rules
              </xsl:otherwise>
            </xsl:choose>
	  </xsl:if>
          remain<xsl:if test="count($pairs) = 1 and string($rules) = 'null'">s</xsl:if>.
        </xsl:when>
        <xsl:otherwise>
	  <xsl:choose>
	    <xsl:when test="string($rules) != 'null' and count($rules) &gt; 0">
              all pairs could be removed, but the
              rule<xsl:if test="count($rules) &gt; 1">s</xsl:if>
              <xsl:apply-templates select="$rules/.."/>
              remain<xsl:if test="count($rules) &gt; 1">s</xsl:if>.
	    </xsl:when>
	    <xsl:when test="string($rules) != 'null'">
              all pairs and rules could be removed.
	    </xsl:when>
	    <xsl:otherwise>
	      all pairs could be removed.
	    </xsl:otherwise>
	  </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
      <p>
        <xsl:apply-templates select="$proof">
          <xsl:with-param name="indent" select="concat($indent,'.1')"/>
        </xsl:apply-templates>
      </p>
    </xsl:template>
    
    <xsl:template match="ceta:simpleProjection">
      <xsl:choose>
        <xsl:when test="count(ceta:projEntry) &gt; 1">
          <p>
          <table>
          <xsl:for-each select="ceta:projEntry">
          <tr>
            <td align="right">&#960;(<xsl:apply-templates select="ceta:name"/>)</td>
            <td align="center">=</td>
            <td align="left"><xsl:value-of select="ceta:argument"/></td>
          </tr>
          </xsl:for-each>
          </table>
          </p>
        </xsl:when>
        <xsl:otherwise>
            &#960;(<xsl:apply-templates select="ceta:projEntry/ceta:name"/>)
            =
            <xsl:value-of select="ceta:projEntry/ceta:argument"/>,
        </xsl:otherwise>
      </xsl:choose>
    </xsl:template>
    
    <xsl:template match="ceta:spscProc">
        <xsl:param name="indent"/>
        <h3><xsl:value-of select="$indent"/> Subterm Criterion Processor</h3>
	<xsl:call-template name="ceta:ProofStep">
	  <xsl:with-param name="indent" select="$indent"/>
	  <xsl:with-param name="name">simple projection</xsl:with-param>
	  <xsl:with-param name="justification" select="ceta:simpleProjection"/>
	  <xsl:with-param name="pairs" select="ceta:dps/ceta:rules/*"/>
	  <xsl:with-param name="proof" select="*[3]"/>
	</xsl:call-template>
    </xsl:template>

    <xsl:template match="ceta:redPairProc">
        <xsl:param name="indent"/>
        <h3><xsl:value-of select="$indent"/> Reduction Pair Processor</h3>
	<xsl:call-template name="ceta:ProofStep">
	  <xsl:with-param name="indent" select="$indent"/>
	  <xsl:with-param name="justification" select="ceta:redPair"/>
	  <xsl:with-param name="pairs" select="ceta:dps/ceta:rules/*"/>
	  <xsl:with-param name="proof" select="*[3]"/>
	</xsl:call-template>
    </xsl:template>

    <xsl:template match="ceta:redPairUrProc">
      <xsl:param name="indent"/>
      <h3><xsl:value-of select="$indent"/> Reduction Pair Processor with Usable Rules</h3>
      <xsl:call-template name="ceta:ProofStep">
        <xsl:with-param name="indent" select="$indent"/>
        <xsl:with-param name="justification" select="ceta:redPair"/>
        <xsl:with-param name="pairs" select="ceta:dps/ceta:rules/*"/>
        <xsl:with-param name="urules" select="ceta:usableRules/ceta:rules/*"/>
        <xsl:with-param name="proof" select="*[4]"/>
      </xsl:call-template>
    </xsl:template>
        
    <xsl:template match="ceta:monoRedPairProc">
      <xsl:param name="indent"/>
      <h3><xsl:value-of select="$indent"/> Monotonic Reduction Pair Processor</h3>
      <xsl:call-template name="ceta:ProofStep">
        <xsl:with-param name="indent" select="$indent"/>
        <xsl:with-param name="justification" select="ceta:redPair"/>
        <xsl:with-param name="pairs" select="ceta:dps/ceta:rules/*"/>
	<xsl:with-param name="rules" select="ceta:trs/ceta:rules/*"/>
        <xsl:with-param name="proof" select="*[4]"/>
      </xsl:call-template>
  </xsl:template>

    <xsl:template match="ceta:monoRedPairUrProc">
      <xsl:param name="indent"/>
      <h3><xsl:value-of select="$indent"/> Monotonic Reduction Pair Processor with Usable Rules</h3>
      <xsl:call-template name="ceta:ProofStep">
        <xsl:with-param name="indent" select="$indent"/>
        <xsl:with-param name="justification" select="ceta:redPair"/>
        <xsl:with-param name="pairs" select="ceta:dps/ceta:rules/*"/>
	<xsl:with-param name="urules" select="ceta:usableRules/ceta:rules/*"/>
	<xsl:with-param name="rules" select="ceta:trs/ceta:rules/*"/>
        <xsl:with-param name="proof" select="*[5]"/>
      </xsl:call-template>
    </xsl:template>
    
    <xsl:template match="ceta:pIsEmpty">
        <xsl:param name="indent"/>
        <h3><xsl:value-of select="$indent"/> P is empty </h3>
        <p>There are no pairs anymore.</p>
    </xsl:template>
    
    <xsl:template match="ceta:ruleRemoval">
      <xsl:param name="indent"/>
      <h3><xsl:value-of select="$indent"/> Rule Removal Processor</h3>
      Using the
      <xsl:apply-templates select="ceta:redPair"/>                
      <xsl:choose>
        <xsl:when test="count(ceta:trs/ceta:rules/*) &gt; 0">
          the
          rule<xsl:if test="count(ceta:trs/ceta:rules/*) &gt; 1">s</xsl:if> 
          <xsl:apply-templates select="ceta:trs/ceta:rules/*/.."/>
          remain<xsl:if test="count(ceta:trs/ceta:rules/*) = 1">s</xsl:if>.
        </xsl:when>
        <xsl:otherwise>
          all rules could be removed.
        </xsl:otherwise>
      </xsl:choose>
      <p>
        <xsl:apply-templates select="*[3]">
          <xsl:with-param name="indent" select="concat($indent, '.1')"/>
        </xsl:apply-templates>
      </p>
    </xsl:template>
    
    <xsl:template match="ceta:rIsEmpty">
        <xsl:param name="indent"/>
        <h3><xsl:value-of select="$indent"/> R is empty </h3>
        <p>All rules have been removed.</p>
    </xsl:template>
    
    <xsl:template match="ceta:depGraphProc">
        <xsl:param name="indent"/>
        <h3><xsl:value-of select="$indent"/> Dependency Graph Processor</h3>
        <xsl:variable name="all" select="count(ceta:component)"/>
        <xsl:variable name="real" select="count(ceta:component/*) - $all"/>
        <p>The dependency pairs are split into <xsl:value-of select="$real"/>
        component<xsl:if test="$real != 1">s</xsl:if>.</p>
        <xsl:choose>
            <xsl:when test="$real &gt; 0">
                <p><ul>    
                    <xsl:apply-templates select="." mode="iterate">
                        <xsl:with-param name="count" select="1"/>
                        <xsl:with-param name="indent" select="$indent"/>
                        <xsl:with-param name="index" select="1"/>
                        <xsl:with-param name="n" select="$all"/>
                    </xsl:apply-templates>
                </ul></p>        
            </xsl:when>
        </xsl:choose>        
    </xsl:template>
    
    <xsl:template mode="iterate" match="ceta:depGraphProc">
        <xsl:param name="indent"/>
        <xsl:param name="count"/>
        <xsl:param name="index"/>
        <xsl:param name="n"/>
        <xsl:variable name="newindex" select="$index + count(ceta:component[$count]/*) -1"/>
        <xsl:if test="$index != $newindex">
            <li>
                The
                <xsl:choose>
                    <xsl:when test="$index = 1">1<sup>st</sup></xsl:when>
                    <xsl:when test="$index = 2">2<sup>nd</sup></xsl:when>
                    <xsl:when test="$index = 3">3<sup>rd</sup></xsl:when>
                    <xsl:otherwise><xsl:value-of select="$index"/><sup>th</sup></xsl:otherwise>
                </xsl:choose>
                component contains the
                pair<xsl:if test="count(ceta:component[$count]/ceta:dps/ceta:rules/ceta:rule) &gt; 1">s</xsl:if>
                <xsl:apply-templates select="ceta:component[$count]/ceta:dps/*"/>
                <xsl:apply-templates select="ceta:component[$count]/*[2]">
                    <xsl:with-param name="indent" select="concat($indent, '.', $index)"/>   
                </xsl:apply-templates>
            </li>
        </xsl:if>
        <xsl:if test="$count &lt; $n">
            <xsl:apply-templates select="." mode="iterate">
                <xsl:with-param name="indent" select="$indent"/>
                <xsl:with-param name="count" select="$count + 1"/>
                <xsl:with-param name="index" select="$newindex"/>
                <xsl:with-param name="n" select="$n"/>
            </xsl:apply-templates>
        </xsl:if>        
    </xsl:template>
    
    <!-- variables always in red -->
    <xsl:template name="var" match="ceta:var">
        <span class="var"><xsl:value-of select="."/></span>
    </xsl:template>
    
    <!-- symbols in blue, sharp symbols in darkgreen-->
    <xsl:template match="ceta:name">
        <xsl:choose>
            <xsl:when test="@sharp = 'true'">
                <span class="dp_fun">
                <xsl:value-of select="text()"/><sup>#</sup></span>
            </xsl:when>
            <xsl:otherwise>
                <span class="fun"><xsl:value-of select="text()"/></span>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="count(ceta:label) &gt; 0">
        <sub>
            <span class="label">
            <xsl:for-each select="ceta:label">
                <xsl:if test="count(ceta:natural) != 1"><xsl:text>(</xsl:text></xsl:if>
                <xsl:for-each select="ceta:natural">
                    <xsl:apply-templates/>
                    <xsl:if test="position() != last()">,</xsl:if>                    
                </xsl:for-each>
                <xsl:if test="count(ceta:natural) != 1"><xsl:text>)</xsl:text></xsl:if>
                <xsl:if test="position() != last()">,</xsl:if>
            </xsl:for-each>     
            </span>
        </sub>            
        </xsl:if>
    </xsl:template>
    
    
    <xsl:template match="ceta:funapp">
      <xsl:apply-templates select="ceta:name"/>
      <xsl:if test="count(ceta:arg) &gt; 0">
      <xsl:text>(</xsl:text>
      <xsl:for-each select="ceta:arg">
        <xsl:apply-templates/>
        <xsl:if test="position() != last()">,</xsl:if>
      </xsl:for-each>
      <xsl:text>)</xsl:text>
      </xsl:if>
    </xsl:template>
    
    <xsl:template match="ceta:box">
        <span style="color:purple">&#9744;</span>
    </xsl:template>
    
    <xsl:template match="ceta:funContext">
        <xsl:apply-templates select="ceta:name"/>
        <xsl:text>(</xsl:text>
        <xsl:for-each select="ceta:before/*">
            <xsl:apply-templates select="."/><xsl:text>,</xsl:text>
        </xsl:for-each>
        <xsl:apply-templates select="*[3]"/>
        <xsl:for-each select="ceta:after/*">
            <xsl:text>,</xsl:text><xsl:apply-templates select="."/>
        </xsl:for-each>
        <xsl:text>)</xsl:text>
    </xsl:template>
    
    
    <xsl:template match="ceta:rules">
      <xsl:choose>
          <xsl:when test="count(ceta:rule) = 0">
          </xsl:when>
          <xsl:when test="count(ceta:rule) = 1">
            <xsl:text> </xsl:text>
            <xsl:apply-templates select="ceta:rule/ceta:lhs"/>
            &#8594;
            <xsl:apply-templates select="ceta:rule/ceta:rhs"/>
          </xsl:when>
          <xsl:otherwise>
              <p>
              <table>
                  <xsl:attribute name="align">center</xsl:attribute>
                  <xsl:for-each select="ceta:rule">
                      <tr>
                          <td align="right"><xsl:apply-templates select="ceta:lhs"/></td>
                          <td align="center">&#8594;</td>
                          <td align="left"><xsl:apply-templates select="ceta:rhs"/></td>
                      </tr>           
                  </xsl:for-each>
              </table>          
              </p>
          </xsl:otherwise>
      </xsl:choose>
    </xsl:template>
    
    <xsl:template match="ceta:substitution">
        <xsl:choose>
            <xsl:when test="count(ceta:substEntry) = 0">
                &#8709;
            </xsl:when>
            <xsl:otherwise>    
                {<xsl:for-each select="ceta:substEntry">
                    <xsl:apply-templates select="*[1]"/>/<xsl:apply-templates select="*[2]"/>
                    <xsl:if test="last() != position()">, </xsl:if>
                </xsl:for-each>}
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    
</xsl:stylesheet>
